@@ -4,12 +4,21 @@ use {
     crate::{
         buffer::Buffer,
         event_bus::DynEvent,
-        modd::ModeState,
+        modd::{ModeState, OperatorType},
         screen::{NavigateDirection, SplitDirection},
     },
     std::{any::Any, fmt::Debug},
 };
 
+/// Operator context for commands executed in operator-pending mode
+#[derive(Debug, Clone, Copy)]
+pub struct OperatorContext {
+    /// The operator type (Delete, Yank, Change)
+    pub operator: OperatorType,
+    /// Optional repeat count
+    pub count: Option<usize>,
+}
+
 /// Execution context passed to commands
 pub struct ExecutionContext<'a> {
     /// The buffer to operate on
@@ -20,6 +29,8 @@ pub struct ExecutionContext<'a> {
     pub buffer_id: usize,
     /// ID of the window containing the buffer
     pub window_id: usize,
+    /// Operator context (if command is being executed in operator-pending mode)
+    pub operator_context: Option<OperatorContext>,
 }
 
 /// Result of command execution
@@ -135,9 +146,6 @@ pub enum WindowAction {
     FocusDirection {
         direction: NavigateDirection,
     },
-    MoveDirection {
-        direction: NavigateDirection,
-    },
     Resize {
         direction: SplitDirection,
         delta: i16,
@@ -254,6 +262,17 @@ pub enum CommandLineAction {
     Execute,
     /// Cancel command line mode
     Cancel,
+    /// Trigger completion (Tab key)
+    Complete,
+    /// Select previous completion (Shift-Tab)
+    CompletePrev,
+    /// Apply the selected completion text
+    ApplyCompletion {
+        /// Text to insert
+        text: String,
+        /// Position where to start replacing
+        replace_start: usize,
+    },
 }
 
 /// The core command trait - all commands must implement this
@@ -310,6 +329,15 @@ pub trait CommandTrait: Debug + Send + Sync {
     fn is_text_modifying(&self) -> bool {
         false
     }
+
+    /// The mode to transition to after this command executes, if any
+    ///
+    /// Commands that change modes should override this to return `Some(ModeState)`.
+    /// This is used by the command handler to update mode immediately before dispatch,
+    /// avoiding race conditions with the runtime's watch channel.
+    fn resulting_mode(&self) -> Option<ModeState> {
+        None
+    }
 }
 
 impl Clone for Box<dyn CommandTrait> {
