@@ -2,6 +2,9 @@
 //!
 //! Defines how windows get their content - from file buffers
 //! or plugin-provided buffers.
+//!
+//! Note: Scroll position (`buffer_anchor`) was moved to `Window::viewport.scroll`
+//! as part of the Window refactoring to group viewport-related state.
 
 mod provider;
 
@@ -9,21 +12,15 @@ pub use provider::{BufferContext, InputResult, PluginBufferProvider};
 
 use std::sync::Arc;
 
-use crate::screen::window::Anchor;
-
 /// Source of content for a window
 #[derive(Clone)]
 pub enum WindowContentSource {
     /// File-backed buffer (traditional editor window)
-    FileBuffer {
-        buffer_id: usize,
-        buffer_anchor: Anchor,
-    },
+    FileBuffer { buffer_id: usize },
 
     /// Plugin-owned buffer (virtual buffer for explorer, LSP, terminal)
     PluginBuffer {
         buffer_id: usize,
-        buffer_anchor: Anchor,
         provider: Arc<dyn PluginBufferProvider>,
     },
 }
@@ -31,22 +28,13 @@ pub enum WindowContentSource {
 impl std::fmt::Debug for WindowContentSource {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            Self::FileBuffer {
-                buffer_id,
-                buffer_anchor,
-            } => f
+            Self::FileBuffer { buffer_id } => f
                 .debug_struct("FileBuffer")
                 .field("buffer_id", buffer_id)
-                .field("buffer_anchor", buffer_anchor)
                 .finish(),
-            Self::PluginBuffer {
-                buffer_id,
-                buffer_anchor,
-                ..
-            } => f
+            Self::PluginBuffer { buffer_id, .. } => f
                 .debug_struct("PluginBuffer")
                 .field("buffer_id", buffer_id)
-                .field("buffer_anchor", buffer_anchor)
                 .finish(),
         }
     }
