@@ -9,8 +9,10 @@ pub use {count_parser::CountParser, dispatcher::Dispatcher, key_parser::key_to_s
 use {
     crate::{
         bind::{CommandRef, KeyMap, KeyMapInner},
-        command::traits::OperatorMotionAction,
-        event::{InnerEvent, KeyEvent, Subscribe, VisualTextObjectAction},
+        command::{CommandRegistry, traits::OperatorMotionAction},
+        event::{RuntimeEvent, ScopedKeyEvent, Subscribe, VisualTextObjectAction},
+        event_bus::EventScope,
+        interactor::InteractorRegistry,
         keystroke::{KeyNotationFormat, KeySequence, Keystroke},
         modd::{ModeState, OperatorType, SubMode},
         motion::Motion,
@@ -19,13 +21,13 @@ use {
             WordTextObject, WordType,
         },
     },
-    std::time::Duration,
+    std::{sync::Arc, time::Duration},
     tokio::sync::{broadcast::Receiver, mpsc::Sender, watch},
 };
 
 /// Handler that translates key events to commands based on current mode
 pub struct CommandHandler {
-    key_event_rx: Option<Receiver<KeyEvent>>,
+    key_event_rx: Option<Receiver<ScopedKeyEvent>>,
     keymap: KeyMap,
     /// Watch receiver for mode changes from Runtime
     mode_rx: watch::Receiver<ModeState>,
@@ -36,10 +38,14 @@ pub struct CommandHandler {
     dispatcher: Dispatcher,
     /// Track when mode was locally changed (avoids race condition with `mode_rx` sync)
     mode_locally_changed: bool,
+    /// Interactor registry for input behavior lookup
+    interactor_registry: Arc<InteractorRegistry>,
+    /// Current scope for event tracking (set per-key, passed to dispatcher)
+    current_scope: Option<EventScope>,
 }
 
-impl Subscribe<KeyEvent> for CommandHandler {
-    fn subscribe(&mut self, rx: Receiver<KeyEvent>) {
+impl Subscribe<ScopedKeyEvent> for CommandHandler {
+    fn subscribe(&mut self, rx: Receiver<ScopedKeyEvent>) {
         self.key_event_rx = Some(rx);
     }
 }
@@ -47,9 +53,11 @@ impl Subscribe<KeyEvent> for CommandHandler {
 impl CommandHandler {
     #[must_use]
     pub fn new(
-        tx: Sender<InnerEvent>,
+        tx: Sender<RuntimeEvent>,
         mode_rx: watch::Receiver<ModeState>,
         keymap: KeyMap,
+        command_registry: Arc<CommandRegistry>,
+        interactor_registry: Arc<InteractorRegistry>,
     ) -> Self {
         let initial_mode = mode_rx.borrow().clone();
         Self {
@@ -59,8 +67,10 @@ impl CommandHandler {
             local_mode: initial_mode,
             pending_keys: KeySequence::new(),
             count_parser: CountParser::new(),
-            dispatcher: Dispatcher::new(tx, 0, 0),
+            dispatcher: Dispatcher::new(tx, 0, 0, command_registry),
             mode_locally_changed: false,
+            interactor_registry,
+            current_scope: None,
         }
     }
 
@@ -75,8 +85,24 @@ impl CommandHandler {
     }
 
     /// Lookup a binding with fallback support
+    /// Also checks runtime mode if different from local mode (handles race conditions
+    /// where focus change hasn't propagated to `local_mode` yet)
     fn lookup_binding(&self, keys: &KeySequence) -> Option<&KeyMapInner> {
-        self.keymap.lookup_binding(&self.local_mode, keys)
+        // First try local mode
+        if let Some(inner) = self.keymap.lookup_binding(&self.local_mode, keys) {
+            return Some(inner);
+        }
+
+        // If local mode failed and runtime mode is different, try runtime mode
+        // This is a defensive fallback for edge cases where local_mode hasn't synced yet
+        {
+            let runtime_mode = self.mode_rx.borrow();
+            if runtime_mode.interactor_id != self.local_mode.interactor_id {
+                return self.keymap.lookup_binding(&runtime_mode, keys);
+            }
+        }
+
+        None
     }
 
     /// Check if keys are a valid prefix (with fallback support)
@@ -383,24 +409,48 @@ impl CommandHandler {
                     // Key event received
                     result = rx.recv() => {
                         match result {
-                            Ok(event) => {
+                            Ok(mut scoped_event) => {
+                                // Extract key and scope from ScopedKeyEvent
+                                let event = scoped_event.key;
+                                // Store scope for this key - will be consumed by dispatcher or
+                                // decremented at end of processing if not dispatched
+                                self.current_scope = scoped_event.take_scope();
+
                                 let key_start = std::time::Instant::now();
                                 let key_str = key_to_string(&event);
                                 if key_str.is_empty() {
+                                    // Decrement scope - no dispatch for empty key
+                                    if let Some(scope) = self.current_scope.take() {
+                                        scope.decrement();
+                                    }
                                     continue;
                                 }
                                 // Convert KeyEvent to Keystroke for pending_keys
                                 let keystroke = Keystroke::from(&event);
 
-                                tracing::debug!("[RTT] CommandHandler: key={} received at {:?}", key_str, key_start);
+                                tracing::trace!("[RTT] CommandHandler: key={} received at {:?}", key_str, key_start);
 
                                 // Sync local mode from Runtime's watch channel
-                                // This catches mode changes initiated by Runtime (e.g., explorer focus)
-                                // Don't overwrite if in a handler-initiated transient state (OperatorPending)
-                                // or if mode was locally changed and runtime hasn't caught up yet
-                                if !self.local_mode.is_operator_pending() {
-                                    let runtime_mode = self.mode_rx.borrow().clone();
-                                    if self.mode_locally_changed {
+                                // This catches mode changes initiated by Runtime (e.g., explorer focus, plugin mode changes)
+                                // Allow syncing even from OperatorPending if runtime mode is different
+                                // (e.g., when plugin changes from OperatorPending to Interactor)
+                                let runtime_mode = self.mode_rx.borrow().clone();
+                                let should_sync = runtime_mode != self.local_mode;
+
+                                // Clear stale flag when runtime caught up
+                                if !should_sync && self.mode_locally_changed {
+                                    self.mode_locally_changed = false;
+                                }
+
+                                if should_sync {
+                                    // If interactor changed, always sync (focus change initiated by runtime)
+                                    let interactor_changed = runtime_mode.interactor_id != self.local_mode.interactor_id;
+
+                                    if interactor_changed {
+                                        // Focus change - always sync and clear mode_locally_changed flag
+                                        self.local_mode = runtime_mode;
+                                        self.mode_locally_changed = false;
+                                    } else if self.mode_locally_changed {
                                         // Check if runtime has caught up with our local change
                                         if runtime_mode == self.local_mode {
                                             self.mode_locally_changed = false;
@@ -419,6 +469,10 @@ impl CommandHandler {
                                     self.dispatcher
                                         .send_pending_keys(self.pending_display())
                                         .await;
+                                    // Decrement scope - no dispatch for count digit
+                                    if let Some(scope) = self.current_scope.take() {
+                                        scope.decrement();
+                                    }
                                     continue;
                                 }
 
@@ -452,7 +506,12 @@ impl CommandHandler {
                                                 let normal = ModeState::normal();
                                                 self.set_local_mode(normal.clone());
                                                 self.mode_locally_changed = true;
-                                                self.dispatcher.update_mode(normal).await;
+                                                self.dispatcher.update_mode(normal, self.current_scope.take()).await;
+                                            } else {
+                                                // Decrement scope - no dispatch for non-op-pending Escape
+                                                if let Some(scope) = self.current_scope.take() {
+                                                    scope.decrement();
+                                                }
                                             }
                                             continue;
                                         }
@@ -460,64 +519,68 @@ impl CommandHandler {
                                     }
                                 }
 
-                                // Handle backspace in Normal/Visual/Explorer modes
-                                if key_str == "Backspace" {
-                                    let mode = self.current_mode();
-                                    if Self::is_backspace_editable_mode(mode) {
-                                        if !self.pending_keys.is_empty() {
-                                            // Remove last keystroke from pending
-                                            self.pending_keys.pop();
+                                // NOTE: Backspace handling moved to AFTER the normal keybinding lookup.
+                                // This ensures that plugins (like explorer) that bind Backspace get
+                                // their bindings checked first, before applying editor-specific behavior.
+                                // The editor-specific "edit pending keys" is now a FALLBACK in the
+                                // "no command found" section at line ~710.
+
+                                // Handle operator-pending mode (d, y, c + motion or text object)
+                                // BUT: First check if there's a registered command for this key
+                                // This allows plugin commands (like jump) to override built-in motions
+                                let mode = self.current_mode();
+                                if mode.is_operator_pending() {
+                                    // Peek at command binding to see if a plugin command is registered
+                                    let (cmd_peek, _) = self.lookup_command_no_push(&key_str);
+                                    let has_registered_command = cmd_peek.is_some();
+
+                                    // Only use built-in motion handling if NO plugin command is registered
+                                    if !has_registered_command {
+                                        let count = self.count_parser.peek();
+                                        let (action, should_wait) = self.handle_operator_pending(
+                                            &key_str,
+                                            count,
+                                            &self.pending_keys,
+                                        );
+                                        if let Some(action) = action {
+                                            tracing::debug!(?action, "Operator-pending action detected");
+                                            self.pending_keys.clear();
+                                            self.count_parser.take(); // Consume count
+                                            // Change actions enter Insert mode, others return to Normal
+                                            let is_change_action = matches!(
+                                                &action,
+                                                OperatorMotionAction::Change { .. }
+                                                    | OperatorMotionAction::ChangeTextObject { .. }
+                                                    | OperatorMotionAction::ChangeWordTextObject { .. }
+                                                    | OperatorMotionAction::ChangeSemanticTextObject { .. }
+                                            );
+                                            let new_mode = if is_change_action {
+                                                ModeState::insert()
+                                            } else {
+                                                ModeState::normal()
+                                            };
+                                            self.set_local_mode(new_mode);
+                                            self.mode_locally_changed = true;
+                                            self.dispatcher.send_operator_motion(action, self.current_scope.take()).await;
                                             self.dispatcher
                                                 .send_pending_keys(self.pending_display())
                                                 .await;
+                                            continue;
+                                        }
+                                        if should_wait {
+                                            // Waiting for text object delimiter (i/a pressed)
+                                            self.pending_keys.push(keystroke.clone());
+                                            self.dispatcher
+                                                .send_pending_keys(self.pending_display())
+                                                .await;
+                                            // Decrement scope - no dispatch while waiting for text object
+                                            if let Some(scope) = self.current_scope.take() {
+                                                scope.decrement();
+                                            }
+                                            continue;
                                         }
-                                        // In Normal/Visual/Explorer, ignore backspace (don't add to pending)
-                                        continue;
-                                    }
-                                }
-
-                                // Handle operator-pending mode (d, y, c + motion or text object)
-                                let mode = self.current_mode();
-                                if mode.is_operator_pending() {
-                                    let count = self.count_parser.peek();
-                                    let (action, should_wait) = self.handle_operator_pending(
-                                        &key_str,
-                                        count,
-                                        &self.pending_keys,
-                                    );
-                                    if let Some(action) = action {
-                                        tracing::debug!(?action, "Operator-pending action detected");
-                                        self.pending_keys.clear();
-                                        self.count_parser.take(); // Consume count
-                                        // Change actions enter Insert mode, others return to Normal
-                                        let is_change_action = matches!(
-                                            &action,
-                                            OperatorMotionAction::Change { .. }
-                                                | OperatorMotionAction::ChangeTextObject { .. }
-                                                | OperatorMotionAction::ChangeWordTextObject { .. }
-                                                | OperatorMotionAction::ChangeSemanticTextObject { .. }
-                                        );
-                                        let new_mode = if is_change_action {
-                                            ModeState::insert()
-                                        } else {
-                                            ModeState::normal()
-                                        };
-                                        self.set_local_mode(new_mode);
-                                        self.mode_locally_changed = true;
-                                        self.dispatcher.send_operator_motion(action).await;
-                                        self.dispatcher
-                                            .send_pending_keys(self.pending_display())
-                                            .await;
-                                        continue;
-                                    }
-                                    if should_wait {
-                                        // Waiting for text object delimiter (i/a pressed)
-                                        self.pending_keys.push(keystroke.clone());
-                                        self.dispatcher
-                                            .send_pending_keys(self.pending_display())
-                                            .await;
-                                        continue;
                                     }
+                                    // If has_registered_command, fall through to normal command dispatch
                                 }
 
                                 // Handle visual mode text object selection (viw, vi(, vif)
@@ -531,7 +594,7 @@ impl CommandHandler {
                                         tracing::debug!(?action, "Visual text object selection detected");
                                         self.pending_keys.clear();
                                         self.count_parser.take(); // Consume count
-                                        self.dispatcher.send_visual_text_object(action).await;
+                                        self.dispatcher.send_visual_text_object(action, self.current_scope.take()).await;
                                         self.dispatcher
                                             .send_pending_keys(self.pending_display())
                                             .await;
@@ -543,6 +606,10 @@ impl CommandHandler {
                                         self.dispatcher
                                             .send_pending_keys(self.pending_display())
                                             .await;
+                                        // Decrement scope - no dispatch while waiting for text object
+                                        if let Some(scope) = self.current_scope.take() {
+                                            scope.decrement();
+                                        }
                                         continue;
                                     }
                                 }
@@ -550,13 +617,13 @@ impl CommandHandler {
                                 // Handle single-character input in modes that accept char input
                                 // Route via TextInputEvent instead of creating inline commands
                                 let mode = self.current_mode();
-                                if mode.accepts_char_input()
+                                if mode.accepts_char_input_with(&self.interactor_registry)
                                     && key_str.len() == 1
                                     && let Some(c) = key_str.chars().next()
                                 {
                                     // Single printable character - route through focus system
                                     self.pending_keys.clear();
-                                    self.dispatcher.send_focus_insert_char(c).await;
+                                    self.dispatcher.send_focus_insert_char(c, self.current_scope.take()).await;
                                     self.dispatcher
                                         .send_pending_keys(self.pending_display())
                                         .await;
@@ -566,10 +633,11 @@ impl CommandHandler {
                                 // Handle Backspace in modes that accept char input
                                 // Route via TextInputEvent for text input handling
                                 let mode = self.current_mode();
-                                if mode.accepts_char_input() && key_str == "Backspace"
+                                if mode.accepts_char_input_with(&self.interactor_registry)
+                                    && key_str == "Backspace"
                                 {
                                     self.pending_keys.clear();
-                                    self.dispatcher.send_focus_delete_backward().await;
+                                    self.dispatcher.send_focus_delete_backward(self.current_scope.take()).await;
                                     self.dispatcher
                                         .send_pending_keys(self.pending_display())
                                         .await;
@@ -590,7 +658,7 @@ impl CommandHandler {
 
                                     // Check for mode change commands and update local mode immediately
                                     // This avoids race conditions with the Runtime's watch channel
-                                    if let Some(new_mode) = Dispatcher::mode_for_command(cmd) {
+                                    if let Some(new_mode) = self.dispatcher.mode_for_command(cmd) {
                                         tracing::debug!(
                                             "[MODE] Immediate mode update: {:?} -> {:?}",
                                             self.current_mode().sub_mode,
@@ -598,28 +666,48 @@ impl CommandHandler {
                                         );
                                         self.set_local_mode(new_mode.clone());
                                         self.mode_locally_changed = true;
-                                        self.dispatcher.update_mode(new_mode).await;
+                                        // Mode change doesn't need scope - dispatch gets the scope
+                                        self.dispatcher.update_mode(new_mode, None).await;
                                     }
 
-                                    // Dispatch the command with count
+                                    // Dispatch the command with count (consumes scope)
                                     let count = self.count_parser.take();
-                                    self.dispatcher.dispatch(cmd.clone(), count).await;
+                                    self.dispatcher.dispatch(cmd.clone(), count, self.current_scope.take()).await;
 
                                     // Clear display after command execution
                                     self.dispatcher
                                         .send_pending_keys(self.pending_display())
                                         .await;
                                 } else {
-                                    // No keybinding found - handle Tab fallback in insert mode
+                                    // No keybinding found - handle fallbacks
                                     let mode = self.current_mode();
-                                    if mode.is_insert() && key_str == "Tab" {
+
+                                    // Backspace fallback: Edit pending keys in editor Normal/Visual mode
+                                    // This is checked AFTER keybinding lookup, so plugin bindings take precedence
+                                    if key_str == "Backspace" && Self::is_backspace_editable_mode(mode) {
+                                        // Only in editor mode, remove last keystroke from pending
+                                        let is_editor = mode.interactor_id.0 == "editor";
+                                        if is_editor && !self.pending_keys.is_empty() {
+                                            self.pending_keys.pop();
+                                            self.dispatcher
+                                                .send_pending_keys(self.pending_display())
+                                                .await;
+                                        }
+                                        // For non-editor modes with no binding, just ignore the Backspace
+                                    } else if mode.is_insert() && key_str == "Tab" {
+                                        // Tab fallback in insert mode
                                         self.pending_keys.clear();
-                                        self.dispatcher.send_focus_insert_char('\t').await;
+                                        self.dispatcher.send_focus_insert_char('\t', self.current_scope.take()).await;
                                         self.dispatcher
                                             .send_pending_keys(self.pending_display())
                                             .await;
                                     }
                                 }
+
+                                // Decrement scope if it wasn't consumed by a dispatch
+                                if let Some(scope) = self.current_scope.take() {
+                                    scope.decrement();
+                                }
                             }
                             Err(e) => {
                                 tracing::debug!(error = %e, "Key event channel closed");
