@@ -20,6 +20,7 @@ pub mod compositor;
 pub mod config;
 pub mod constants;
 pub mod content;
+pub mod context_provider;
 pub mod event;
 pub mod filetype;
 pub mod folding;
@@ -39,9 +40,11 @@ pub mod render;
 pub mod runtime;
 pub mod screen;
 pub mod settings;
+pub mod sign;
 // settings_menu is now a standalone plugin: reovim-plugin-settings-menu
 pub mod style;
 pub mod syntax;
 pub mod testing;
 pub mod textobject;
 pub mod types;
+pub mod ui;
