@@ -24,6 +24,7 @@ use crate::{
             CursorLeftCommand,
             CursorLineEndCommand,
             CursorLineStartCommand,
+            CursorMatchingBracketCommand,
             CursorRightCommand,
             CursorUpCommand,
             CursorWordBackwardCommand,
@@ -96,12 +97,11 @@ use crate::{
         },
         id::CommandId,
     },
-    display::{DisplayInfo, EditModeKey, SubModeKey},
     event_bus::{
         EventBus, EventResult,
         core_events::{
             RequestSetIndentGuide, RequestSetLineNumbers, RequestSetRelativeLineNumbers,
-            RequestSetScrollbar, RequestSetTheme,
+            RequestSetScrollbar, RequestSetSignColumn, RequestSetTheme,
         },
     },
     keys,
@@ -111,6 +111,7 @@ use crate::{
         OptionValue, RegisterOption, RegisterSettingSection,
     },
     plugin::{Plugin, PluginContext, PluginId, PluginStateRegistry},
+    screen::window::SignColumnMode,
 };
 
 /// Core editor functionality plugin
@@ -153,7 +154,6 @@ impl Plugin for CorePlugin {
         self.register_buffer_commands(ctx);
         self.register_system_commands(ctx);
         self.register_window_mode_commands(ctx);
-        self.register_builtin_displays(ctx);
         self.register_default_keybindings(ctx);
     }
 
@@ -206,6 +206,20 @@ impl CorePlugin {
                         needs_render = true;
                     }
                 }
+                "signcolumn" => {
+                    if let OptionValue::String(value) = &event.new_value {
+                        // Parse signcolumn value: "auto", "yes", "no", "number"
+                        let mode = match value.as_str() {
+                            "yes" => SignColumnMode::Yes(2),
+                            "no" => SignColumnMode::No,
+                            "number" => SignColumnMode::Number,
+                            // "auto" and any invalid value default to Auto
+                            _ => SignColumnMode::Auto,
+                        };
+                        ctx.emit(RequestSetSignColumn { mode });
+                        needs_render = true;
+                    }
+                }
                 "indentguide" => {
                     if let OptionValue::Bool(enabled) = &event.new_value {
                         ctx.emit(RequestSetIndentGuide { enabled: *enabled });
@@ -235,6 +249,7 @@ impl CorePlugin {
         let _ = ctx.register_command(CursorWordForwardCommand);
         let _ = ctx.register_command(CursorWordBackwardCommand);
         let _ = ctx.register_command(CursorWordEndCommand);
+        let _ = ctx.register_command(CursorMatchingBracketCommand);
         let _ = ctx.register_command(GotoFirstLineCommand);
         let _ = ctx.register_command(GotoLastLineCommand);
     }
@@ -339,49 +354,6 @@ impl CorePlugin {
         let _ = ctx.register_command(WindowModeEqualizeCommand);
     }
 
-    /// Register built-in display strings and icons for core modes
-    fn register_builtin_displays(&self, ctx: &mut PluginContext) {
-        // === Editor modes ===
-        ctx.register_component_mode_display(
-            ComponentId::EDITOR,
-            EditModeKey::Normal,
-            DisplayInfo::new(" NORMAL ", "󰆾 "),
-        );
-        ctx.register_component_mode_display(
-            ComponentId::EDITOR,
-            EditModeKey::Insert,
-            DisplayInfo::new(" INSERT ", "󰏫 "),
-        );
-        ctx.register_component_mode_display(
-            ComponentId::EDITOR,
-            EditModeKey::InsertReplace,
-            DisplayInfo::new(" REPLACE ", "󰏫 "),
-        );
-        ctx.register_component_mode_display(
-            ComponentId::EDITOR,
-            EditModeKey::VisualChar,
-            DisplayInfo::new(" VISUAL ", "󰒉 "),
-        );
-        ctx.register_component_mode_display(
-            ComponentId::EDITOR,
-            EditModeKey::VisualLine,
-            DisplayInfo::new(" V-LINE ", "󰒉 "),
-        );
-        ctx.register_component_mode_display(
-            ComponentId::EDITOR,
-            EditModeKey::VisualBlock,
-            DisplayInfo::new(" V-BLOCK ", "󰒉 "),
-        );
-
-        // === Sub-modes (core only) ===
-        ctx.register_sub_mode_display(SubModeKey::Command, DisplayInfo::new(" COMMAND ", " "));
-        ctx.register_sub_mode_display(
-            SubModeKey::OperatorPending,
-            DisplayInfo::new(" OPERATOR ", "󰆾 "),
-        );
-        // Note: Plugin sub-modes (Leap, etc.) are registered by their respective plugins
-    }
-
     /// Register default vim-style keybindings for the editor
     #[allow(clippy::too_many_lines)]
     fn register_default_keybindings(&self, ctx: &mut PluginContext) {
@@ -446,6 +418,11 @@ impl CorePlugin {
             keys!['G'],
             CommandRef::Registered(CommandId::new("goto_last_line")),
         );
+        ctx.bind_key_scoped(
+            editor_normal.clone(),
+            keys!['%'],
+            CommandRef::Registered(CommandId::new("cursor_matching_bracket")),
+        );
 
         // === Mode switching ===
         ctx.bind_key_scoped(
@@ -822,6 +799,19 @@ impl CorePlugin {
             .with_display_order(11),
         ));
 
+        // Sign column
+        bus.emit(RegisterOption::new(
+            OptionSpec::new(
+                "signcolumn",
+                "Sign column display ('auto', 'yes', 'no', 'number')",
+                OptionValue::String("yes".into()),
+            )
+            .with_category(OptionCategory::Editor)
+            .with_section("Editor")
+            .with_scope(OptionScope::Window)
+            .with_display_order(12),
+        ));
+
         // Tab width
         bus.emit(RegisterOption::new(
             OptionSpec::new("tabwidth", "Spaces per tab", OptionValue::Integer(4))
@@ -931,5 +921,48 @@ impl CorePlugin {
             .with_section("Window")
             .with_display_order(11),
         ));
+
+        // Virtual text options (diagnostics)
+        bus.emit(RegisterOption::new(
+            OptionSpec::new("virtual_text", "Show inline diagnostics", OptionValue::Bool(true))
+                .with_short("vt")
+                .with_category(OptionCategory::Editor)
+                .with_section("Diagnostics")
+                .with_display_order(40),
+        ));
+
+        bus.emit(RegisterOption::new(
+            OptionSpec::new(
+                "virtual_text_prefix",
+                "Prefix for virtual text",
+                OptionValue::String(String::new()),
+            )
+            .with_category(OptionCategory::Editor)
+            .with_section("Diagnostics")
+            .with_display_order(41),
+        ));
+
+        bus.emit(RegisterOption::new(
+            OptionSpec::new(
+                "virtual_text_max_length",
+                "Maximum virtual text length",
+                OptionValue::Integer(80),
+            )
+            .with_category(OptionCategory::Editor)
+            .with_section("Diagnostics")
+            .with_constraint(OptionConstraint::range(10, 200))
+            .with_display_order(42),
+        ));
+
+        bus.emit(RegisterOption::new(
+            OptionSpec::new(
+                "virtual_text_show",
+                "Which diagnostics to show ('first', 'highest', 'all')",
+                OptionValue::choice("first", vec!["first".into(), "highest".into(), "all".into()]),
+            )
+            .with_category(OptionCategory::Editor)
+            .with_section("Diagnostics")
+            .with_display_order(43),
+        ));
     }
 }
