@@ -8,16 +8,16 @@ use crate::{
     command::{
         CommandTrait,
         traits::{
-            BufferAction, CommandLineAction, CommandResult, DeferredAction, ExecutionContext,
-            FileAction, OperatorMotionAction, TabAction, WindowAction,
+            CommandLineAction, CommandResult, DeferredAction, ExecutionContext,
+            OperatorMotionAction,
         },
     },
     command_line::{ExCommand, SetOption},
     event::{CommandEvent, VisualTextObjectAction},
-    event_bus::BufferModification,
+    event_bus::{BufferModification, ViewportScrolled},
     highlight::Theme,
-    modd::{ComponentId, ModeState, SubMode},
-    screen::{NavigateDirection, Position, SplitDirection},
+    modd::ModeState,
+    screen::Position,
     textobject::SemanticTextObjectSpec,
 };
 
@@ -28,12 +28,28 @@ impl Runtime {
     ///
     /// Emits a `BufferModified` event so the treesitter plugin can handle the reparse.
     pub(crate) fn schedule_treesitter_reparse(&self, buffer_id: usize) {
-        self.event_bus.emit(crate::event_bus::BufferModified {
+        self.emit_event(crate::event_bus::BufferModified {
             buffer_id,
             modification: BufferModification::FullReplace,
         });
     }
 
+    /// Check and emit viewport scroll events
+    ///
+    /// Call this after cursor movement to detect if the viewport needs to scroll.
+    /// Emits `ViewportScrolled` events for any windows that scrolled.
+    pub(crate) fn emit_viewport_scrolls(&mut self) {
+        let scrolls = self.screen.update_viewport_scrolls(&self.buffers);
+        for scroll_info in scrolls {
+            self.emit_event(ViewportScrolled {
+                window_id: scroll_info.window_id,
+                buffer_id: scroll_info.buffer_id,
+                top_line: scroll_info.top_line,
+                bottom_line: scroll_info.bottom_line,
+            });
+        }
+    }
+
     /// Delete a semantic text object using plugin state's text object source
     ///
     /// Returns the deleted text if successful, None if bounds couldn't be found.
@@ -249,7 +265,7 @@ impl Runtime {
                         }
                         ExCommand::Edit { filename } => {
                             self.open_file(&filename);
-                            self.screen.set_editor_buffer(self.active_buffer_id);
+                            self.screen.set_editor_buffer(self.active_buffer_id());
                         }
                         // Window management
                         ExCommand::Split { filename } => {
@@ -281,25 +297,26 @@ impl Runtime {
                         ExCommand::TabPrev => {
                             self.handle_tab_prev();
                         }
-                        // Profile management
-                        ExCommand::ProfileLoad { name } => {
-                            if self.load_profile(&name) {
-                                tracing::info!(profile = %name, "Profile loaded");
+                        // Plugin-registered ex-commands
+                        ExCommand::Plugin { command } => {
+                            if let Some(event) = self.ex_command_registry.dispatch(&command) {
+                                let sender = self.event_bus.sender();
+                                let mut ctx = crate::event_bus::HandlerContext::new(&sender);
+                                self.event_bus.dispatch(&event, &mut ctx);
+
+                                tracing::debug!(command = %command, "Ex-command dispatched via registry");
+
+                                if ctx.render_requested() {
+                                    self.request_render();
+                                }
+                                if ctx.quit_requested() {
+                                    self.command_line.clear();
+                                    return true;
+                                }
+                            } else {
+                                tracing::warn!(command = %command, "Unknown or unregistered ex-command");
                             }
                         }
-                        ExCommand::ProfileSave { name } => {
-                            if self.save_current_as_profile(&name) {
-                                tracing::info!(profile = %name, "Profile saved");
-                            }
-                        }
-                        ExCommand::ProfileList => {
-                            // Telescope handled by plugin via EventBus
-                            tracing::debug!("Profile list command - handled by telescope plugin");
-                        }
-                        ExCommand::Settings => {
-                            // Settings menu handled by plugin via EventBus
-                            tracing::debug!("Settings command - handled by settings-menu plugin");
-                        }
                         ExCommand::Unknown(cmd) => {
                             tracing::warn!(command = %cmd, "Unknown ex-command");
                         }
@@ -312,6 +329,26 @@ impl Runtime {
                 // Just clear the command line, mode change handled by CommandHandler
                 self.command_line.clear();
             }
+            CommandLineAction::Complete => {
+                // Emit event with command line context for plugin to handle
+                let context = self.command_line.completion_context();
+                let registry_commands = self.ex_command_registry.list_commands();
+                self.emit_event(crate::event_bus::core_events::CmdlineCompletionTriggered {
+                    context,
+                    registry_commands,
+                });
+            }
+            CommandLineAction::CompletePrev => {
+                // Emit event for plugin to select previous completion
+                self.emit_event(crate::event_bus::core_events::CmdlineCompletionPrevRequested);
+            }
+            CommandLineAction::ApplyCompletion {
+                text,
+                replace_start,
+            } => {
+                // Apply the completion text to the command line
+                self.command_line.apply_completion(text, *replace_start);
+            }
         }
         false
     }
@@ -343,7 +380,7 @@ impl Runtime {
         // Use active_buffer_id instead of context.buffer_id since the dispatcher
         // doesn't track buffer changes. In a single-window editor, active_buffer_id
         // is the correct buffer to operate on.
-        let buffer_id = self.active_buffer_id;
+        let buffer_id = self.active_buffer_id();
 
         // Get the buffer and execute the command
         if let Some(buffer) = self.buffers.get_mut(&buffer_id) {
@@ -355,11 +392,22 @@ impl Runtime {
             // Capture cursor position BEFORE command execution for CursorMoved event
             let cursor_before = buffer.cur;
 
+            // Extract operator context if in OperatorPending mode
+            let operator_context =
+                if let crate::modd::SubMode::OperatorPending { operator, count } =
+                    self.mode_state.sub_mode
+                {
+                    Some(crate::command::traits::OperatorContext { operator, count })
+                } else {
+                    None
+                };
+
             let mut exec_ctx = ExecutionContext {
                 buffer,
                 count: context.count,
                 buffer_id,
                 window_id: context.window_id,
+                operator_context,
             };
 
             let result = cmd.execute(&mut exec_ctx);
@@ -373,11 +421,13 @@ impl Runtime {
             // Check if cursor moved and emit event
             let cursor_after = exec_ctx.buffer.cur;
             if cursor_before != cursor_after {
-                self.event_bus.emit(crate::event_bus::CursorMoved {
+                self.emit_event(crate::event_bus::CursorMoved {
                     buffer_id,
                     from: (cursor_before.y as usize, cursor_before.x as usize),
                     to: (cursor_after.y as usize, cursor_after.x as usize),
                 });
+                // Check if viewport needs to scroll and emit ViewportScrolled events
+                self.emit_viewport_scrolls();
             }
 
             // Check if command is text-modifying for treesitter reparse
@@ -454,12 +504,15 @@ impl Runtime {
 
                             // Handle paste from specified register
                             // Use active_buffer_id, not context.buffer_id (which is hardcoded to 0 in dispatcher)
-                            let paste_buffer_id = self.active_buffer_id;
+                            let paste_buffer_id = self.active_buffer_id();
                             if let Some(content) = self.registers.get_by_name(register)
                                 && let Some(buf) = self.buffers.get_mut(&paste_buffer_id)
                             {
                                 match content.yank_type {
                                     YankType::Characterwise => {
+                                        // Track start position for animation
+                                        let start_pos = buf.cur;
+
                                         // For characterwise paste:
                                         // p (before=false): paste AFTER cursor (move right by 1 first)
                                         // P (before=true): paste BEFORE cursor (at current position)
@@ -473,10 +526,48 @@ impl Runtime {
                                                 }
                                             }
                                         }
+
                                         buf.insert_text(&content.text);
+
+                                        // Track end position and trigger animation
+                                        let end_pos = buf.cur;
+                                        self.trigger_paste_animation(
+                                            paste_buffer_id,
+                                            start_pos,
+                                            end_pos,
+                                        );
                                     }
                                     YankType::Linewise => {
+                                        // Calculate line range for animation
+                                        let start_line =
+                                            if before { buf.cur.y } else { buf.cur.y + 1 };
+
                                         buf.insert_linewise(&content.text, before);
+
+                                        // Calculate end position
+                                        #[allow(clippy::cast_possible_truncation)]
+                                        let line_count = content.text.lines().count() as u16;
+                                        let end_line = start_line + line_count.saturating_sub(1);
+
+                                        // Get line lengths for column bounds
+                                        let start_col = 0;
+                                        #[allow(clippy::cast_possible_truncation)]
+                                        let end_col = buf
+                                            .contents
+                                            .get(end_line as usize)
+                                            .map_or(0, |line| line.inner.len() as u16);
+
+                                        self.trigger_paste_animation(
+                                            paste_buffer_id,
+                                            crate::screen::Position {
+                                                x: start_col,
+                                                y: start_line,
+                                            },
+                                            crate::screen::Position {
+                                                x: end_col,
+                                                y: end_line,
+                                            },
+                                        );
                                     }
                                 }
                             }
@@ -497,7 +588,7 @@ impl Runtime {
                             );
 
                             // Get current position to skip if jump returns the same position
-                            let current_buf_id = self.active_buffer_id;
+                            let current_buf_id = self.active_buffer_id();
                             let current_pos = self.buffers.get(&current_buf_id).map(|b| b.cur);
 
                             // Try to jump older, skipping current position if needed
@@ -535,7 +626,7 @@ impl Runtime {
                         }
                         DeferredAction::JumpNewer => {
                             // Get current position to skip if jump returns the same position
-                            let current_buf_id = self.active_buffer_id;
+                            let current_buf_id = self.active_buffer_id();
                             let current_pos = self.buffers.get(&current_buf_id).map(|b| b.cur);
 
                             // Try to jump newer, skipping current position if needed
@@ -596,8 +687,16 @@ impl Runtime {
                 CommandResult::EmitEvent(event) => {
                     tracing::info!("Command result: EmitEvent, type={}", event.type_name());
                     // Dispatch the event to the event bus for plugin handling
+                    // Propagate the current scope for proper lifecycle tracking
                     let sender = self.event_bus.sender();
-                    let mut ctx = crate::event_bus::HandlerContext::new(&sender);
+                    let mut ctx = crate::event_bus::HandlerContext::new(&sender)
+                        .with_scope(self.current_scope.clone());
+
+                    // Increment scope before dispatch (will be decremented after)
+                    if let Some(ref scope) = self.current_scope {
+                        scope.increment();
+                    }
+
                     let result = self.event_bus.dispatch(&event, &mut ctx);
                     tracing::info!(
                         "Event dispatched: type={}, result={:?}",
@@ -605,15 +704,21 @@ impl Runtime {
                         result
                     );
 
+                    // Decrement scope after dispatch completes
+                    if let Some(ref scope) = self.current_scope {
+                        scope.decrement();
+                    }
+
                     // Check if any handler requested render or quit
                     if ctx.render_requested() {
                         self.request_render();
                     }
                     if ctx.quit_requested() {
                         // Emit quit - the event loop will handle it
-                        let tx = self.tx.clone();
+                        // HIGH PRIORITY: Kill signal must be processed immediately
+                        let hi_tx = self.hi_tx.clone();
                         tokio::spawn(async move {
-                            let _ = tx.send(crate::event::InnerEvent::KillSignal).await;
+                            let _ = hi_tx.send(crate::event::RuntimeEvent::kill()).await;
                         });
                     }
 
@@ -642,7 +747,7 @@ impl Runtime {
     /// Handle operator + motion action (d/y/c + motion)
     #[allow(clippy::too_many_lines)]
     pub(crate) fn handle_operator_motion(&mut self, action: &OperatorMotionAction) {
-        let buffer_id = self.active_buffer_id;
+        let buffer_id = self.active_buffer_id();
         let mut text_modified = false;
 
         if let Some(buffer) = self.buffers.get_mut(&buffer_id) {
@@ -661,6 +766,8 @@ impl Runtime {
                         self.registers.set_with_type(deleted, yank_type);
                         text_modified = true;
                     }
+                    // Return to normal mode after delete
+                    self.set_mode(ModeState::normal());
                 }
                 OperatorMotionAction::Yank { motion, count } => {
                     use crate::register::YankType;
@@ -686,6 +793,8 @@ impl Runtime {
                         self.registers.set_with_type(yanked, yank_type);
                     }
                     // Yank doesn't modify text
+                    // Return to normal mode after yank
+                    self.set_mode(ModeState::normal());
                 }
                 OperatorMotionAction::Change { motion, count } => {
                     use crate::register::YankType;
@@ -710,6 +819,8 @@ impl Runtime {
                         self.registers.set(deleted);
                         text_modified = true;
                     }
+                    // Return to normal mode after delete
+                    self.set_mode(ModeState::normal());
                 }
                 OperatorMotionAction::YankTextObject { text_object } => {
                     // Get range for visual feedback
@@ -725,6 +836,8 @@ impl Runtime {
                         self.registers.set(yanked);
                     }
                     // Yank doesn't modify text
+                    // Return to normal mode after yank
+                    self.set_mode(ModeState::normal());
                 }
                 OperatorMotionAction::ChangeTextObject { text_object } => {
                     let deleted = buffer.delete_text_object(text_object);
@@ -742,6 +855,8 @@ impl Runtime {
                         self.registers.set(deleted);
                         text_modified = true;
                     }
+                    // Return to normal mode after delete
+                    self.set_mode(ModeState::normal());
                 }
                 OperatorMotionAction::YankSemanticTextObject { text_object } => {
                     if let Some(yanked) = self.yank_semantic_text_object(buffer_id, text_object)
@@ -749,6 +864,8 @@ impl Runtime {
                     {
                         self.registers.set(yanked);
                     }
+                    // Return to normal mode after yank
+                    self.set_mode(ModeState::normal());
                 }
                 OperatorMotionAction::ChangeSemanticTextObject { text_object } => {
                     if let Some(deleted) = self.delete_semantic_text_object(buffer_id, text_object)
@@ -766,6 +883,8 @@ impl Runtime {
                         self.registers.set(deleted);
                         text_modified = true;
                     }
+                    // Return to normal mode after delete
+                    self.set_mode(ModeState::normal());
                 }
                 OperatorMotionAction::YankWordTextObject { text_object } => {
                     let yanked = buffer.yank_word_text_object(text_object);
@@ -773,6 +892,8 @@ impl Runtime {
                         self.registers.set(yanked);
                     }
                     // Yank doesn't modify text
+                    // Return to normal mode after yank
+                    self.set_mode(ModeState::normal());
                 }
                 OperatorMotionAction::ChangeWordTextObject { text_object } => {
                     let deleted = buffer.delete_word_text_object(text_object);
@@ -879,515 +1000,4 @@ impl Runtime {
             _ => {}
         }
     }
-
-    /// Handle window-related deferred actions. Returns true if editor should quit.
-    pub(crate) fn handle_window_action(&mut self, action: &WindowAction) -> bool {
-        match action {
-            WindowAction::SplitHorizontal { filename } => {
-                self.handle_window_split(false, filename.as_ref());
-            }
-            WindowAction::SplitVertical { filename } => {
-                self.handle_window_split(true, filename.as_ref());
-            }
-            WindowAction::Close { force } => {
-                if self.handle_window_close(*force) {
-                    return true;
-                }
-            }
-            WindowAction::CloseOthers => {
-                self.handle_window_only();
-            }
-            WindowAction::FocusDirection { direction } => {
-                self.handle_window_navigate(*direction);
-            }
-            WindowAction::MoveDirection { direction } => {
-                // Window movement (swap windows) - not yet implemented
-                tracing::info!(direction = ?direction, "Window move direction (not yet implemented)");
-            }
-            WindowAction::Resize { direction, delta } => {
-                tracing::info!(direction = ?direction, delta = %delta, "Window resize (not yet implemented)");
-            }
-            WindowAction::Equalize => {
-                self.handle_window_equalize();
-            }
-            WindowAction::SwapDirection { direction } => {
-                self.screen.swap_window(*direction);
-            }
-        }
-
-        // Exit window mode if we're in it
-        self.exit_window_mode_if_active();
-
-        false
-    }
-
-    /// Exit window mode if currently active
-    fn exit_window_mode_if_active(&mut self) {
-        if matches!(
-            self.mode_state.sub_mode,
-            SubMode::Interactor(id) if id == ComponentId::WINDOW
-        ) {
-            self.handle_mode_change(ModeState::normal());
-        }
-    }
-
-    /// Handle tab-related deferred actions. Returns true if editor should quit.
-    pub(crate) fn handle_tab_action(&mut self, action: &TabAction) -> bool {
-        match action {
-            TabAction::New { filename } => {
-                self.handle_tab_new(filename.as_ref());
-            }
-            TabAction::Close => {
-                if self.handle_tab_close() {
-                    return true;
-                }
-            }
-            TabAction::Next => {
-                self.handle_tab_next();
-            }
-            TabAction::Prev => {
-                self.handle_tab_prev();
-            }
-            TabAction::Goto { index } => {
-                self.screen.goto_tab(*index);
-            }
-        }
-        false
-    }
-
-    /// Handle window split command
-    ///
-    /// Saves the current window's cursor before splitting so the new window
-    /// inherits the correct cursor position. See docs/window-buffer.md.
-    pub(crate) fn handle_window_split(&mut self, vertical: bool, filename: Option<&String>) {
-        let start = std::time::Instant::now();
-        tracing::debug!("[SPLIT] handle_window_split START vertical={}", vertical);
-
-        let direction = if vertical {
-            SplitDirection::Vertical
-        } else {
-            SplitDirection::Horizontal
-        };
-
-        // If a filename is provided, open it
-        if let Some(path) = filename {
-            self.open_file(path);
-        }
-
-        // Get the current buffer ID (either the newly opened file or existing buffer)
-        // Use self.active_buffer_id directly since open_file updates it
-        let buffer_id = self.active_buffer_id;
-
-        // CRITICAL: Save the buffer's live cursor to the current window BEFORE splitting.
-        // split_window() reads window.cursor to copy to the new window, so we must
-        // ensure it reflects the current buffer cursor, not a stale saved value.
-        if let Some(window) = self.screen.active_window_mut()
-            && let Some(buffer) = self.buffers.get(&buffer_id)
-        {
-            tracing::debug!(
-                "[SPLIT] SAVE cursor: win={} buffer.cur=({},{}) -> window.cursor at {:?}",
-                window.id,
-                buffer.cur.x,
-                buffer.cur.y,
-                start.elapsed()
-            );
-            window.cursor = buffer.cur;
-            window.desired_col = buffer.desired_col;
-        } else {
-            tracing::warn!("[SPLIT] failed to get active window or buffer!");
-        }
-
-        // Split the window
-        if let Some(new_window_id) = self.screen.split_window(direction) {
-            // Set the buffer for the new window
-            self.screen.set_window_buffer(new_window_id, buffer_id);
-            tracing::debug!(
-                "[SPLIT] created new_win={} buffer={} at {:?}",
-                new_window_id,
-                buffer_id,
-                start.elapsed()
-            );
-        }
-        tracing::debug!("[SPLIT] handle_window_split DONE at {:?}", start.elapsed());
-    }
-
-    /// Handle window close command. Returns true if editor should quit.
-    pub(crate) fn handle_window_close(&mut self, _force: bool) -> bool {
-        // TODO: Check for unsaved changes if force is false
-        self.screen.close_window()
-    }
-
-    /// Handle close all other windows (:only)
-    pub(crate) fn handle_window_only(&mut self) {
-        self.screen.close_other_windows();
-    }
-
-    /// Handle window navigation (focus direction)
-    pub(crate) fn handle_window_navigate(&mut self, direction: NavigateDirection) {
-        let start = std::time::Instant::now();
-        let before_window_id = self.screen.active_window_id();
-
-        // Before navigation: save current buffer cursor to current window
-        if let Some(window) = self.screen.active_window_mut()
-            && let Some(buffer_id) = window.buffer_id()
-            && let Some(buffer) = self.buffers.get(&buffer_id)
-        {
-            tracing::debug!(
-                "[NAV] SAVE cursor: win={} buffer.cur=({},{}) -> window.cursor at {:?}",
-                window.id,
-                buffer.cur.x,
-                buffer.cur.y,
-                start.elapsed()
-            );
-            window.cursor = buffer.cur;
-            window.desired_col = buffer.desired_col;
-        }
-
-        // Navigate to new window
-        self.screen.navigate_window(direction);
-
-        let after_window_id = self.screen.active_window_id();
-        tracing::debug!(
-            "[NAV] navigate {:?}: win {} -> win {:?} at {:?}",
-            direction,
-            before_window_id.unwrap_or(999),
-            after_window_id,
-            start.elapsed()
-        );
-
-        // After navigation: load new window's cursor into buffer and update active_buffer_id
-        if let Some(window) = self.screen.active_window()
-            && let Some(new_buffer_id) = window.buffer_id()
-        {
-            let window_cursor = window.cursor;
-            let window_desired_col = window.desired_col;
-
-            tracing::debug!(
-                "[NAV] LOAD cursor: win={} window.cursor=({},{}) -> buffer.cur at {:?}",
-                window.id,
-                window_cursor.x,
-                window_cursor.y,
-                start.elapsed()
-            );
-
-            // Update active_buffer_id to match the new window's buffer
-            self.active_buffer_id = new_buffer_id;
-
-            // Load window's cursor into buffer
-            if let Some(buffer) = self.buffers.get_mut(&new_buffer_id) {
-                buffer.cur = window_cursor;
-                buffer.desired_col = window_desired_col;
-            }
-        }
-        tracing::debug!("[NAV] handle_window_navigate DONE at {:?}", start.elapsed());
-    }
-
-    /// Handle window equalize
-    pub(crate) fn handle_window_equalize(&mut self) {
-        self.screen.equalize_windows();
-    }
-
-    // === Tab Management Handlers ===
-
-    /// Handle new tab command
-    pub(crate) fn handle_tab_new(&mut self, filename: Option<&String>) {
-        // If a filename is provided, open it
-        if let Some(path) = filename {
-            self.open_file(path);
-        }
-
-        // Get the current buffer ID
-        // Use self.active_buffer_id directly since open_file updates it
-        let buffer_id = self.active_buffer_id;
-
-        let tab_id = self.screen.new_tab(buffer_id);
-        tracing::info!(tab_id = tab_id, buffer_id = buffer_id, "New tab created");
-    }
-
-    /// Handle tab close command. Returns true if editor should quit.
-    pub(crate) fn handle_tab_close(&mut self) -> bool {
-        self.screen.close_tab()
-    }
-
-    /// Handle next tab command (gt)
-    pub(crate) fn handle_tab_next(&mut self) {
-        self.screen.next_tab();
-    }
-
-    /// Handle previous tab command (gT)
-    pub(crate) fn handle_tab_prev(&mut self) {
-        self.screen.prev_tab();
-    }
-
-    // === Buffer Navigation Handlers ===
-
-    /// Handle buffer-related deferred actions
-    pub(crate) fn handle_buffer_action(&mut self, action: &BufferAction) {
-        match action {
-            BufferAction::Prev => {
-                if let Some(prev_id) = self.prev_buffer_id() {
-                    self.switch_buffer(prev_id);
-                    // Update window's buffer
-                    if let Some(window_id) = self.screen.active_window_id() {
-                        self.screen.set_window_buffer(window_id, prev_id);
-                    }
-                }
-            }
-            BufferAction::Next => {
-                if let Some(next_id) = self.next_buffer_id() {
-                    self.switch_buffer(next_id);
-                    if let Some(window_id) = self.screen.active_window_id() {
-                        self.screen.set_window_buffer(window_id, next_id);
-                    }
-                }
-            }
-            BufferAction::Delete { force: _ } => {
-                // TODO: Check modified state if !force
-                let buffer_id = self.active_buffer_id;
-                self.close_buffer(buffer_id);
-                // Update window to show the new active buffer
-                if let Some(window_id) = self.screen.active_window_id() {
-                    self.screen
-                        .set_window_buffer(window_id, self.active_buffer_id);
-                }
-            }
-        }
-        self.request_render();
-    }
-
-    // ========================================================================
-    // File Operations
-    // ========================================================================
-
-    /// Handle file operations from plugins
-    ///
-    /// This provides a way for plugins to request file operations that need
-    /// Runtime access (e.g., opening files into buffers, creating files).
-    pub(crate) fn handle_file_action(&mut self, action: &FileAction) {
-        match action {
-            FileAction::Open { path } => {
-                self.open_file(path);
-                self.screen.set_editor_buffer(self.active_buffer_id);
-                // Switch focus to editor if a plugin has focus
-                if self.screen.has_plugin_focus() {
-                    self.screen.focus_editor();
-                    self.set_mode(ModeState::normal());
-                }
-            }
-            FileAction::Create { path, is_dir } => {
-                let result = if *is_dir {
-                    std::fs::create_dir_all(path)
-                } else {
-                    // Create parent directories if needed, then create file
-                    if let Some(parent) = std::path::Path::new(path).parent() {
-                        let _ = std::fs::create_dir_all(parent);
-                    }
-                    std::fs::File::create(path).map(|_| ())
-                };
-                match result {
-                    Ok(()) => {
-                        tracing::info!(path = %path, is_dir = is_dir, "File/directory created");
-                    }
-                    Err(e) => {
-                        tracing::error!(path = %path, error = %e, "Failed to create file/directory");
-                    }
-                }
-            }
-            FileAction::Delete { path } => {
-                let path = std::path::Path::new(path);
-                let result = if path.is_dir() {
-                    std::fs::remove_dir_all(path)
-                } else {
-                    std::fs::remove_file(path)
-                };
-                match result {
-                    Ok(()) => {
-                        tracing::info!(path = %path.display(), "File/directory deleted");
-                    }
-                    Err(e) => {
-                        tracing::error!(path = %path.display(), error = %e, "Failed to delete");
-                    }
-                }
-            }
-            FileAction::Rename { from, to } => match std::fs::rename(from, to) {
-                Ok(()) => {
-                    tracing::info!(from = %from, to = %to, "File/directory renamed");
-                }
-                Err(e) => {
-                    tracing::error!(from = %from, to = %to, error = %e, "Failed to rename");
-                }
-            },
-        }
-        self.request_render();
-    }
-
-    // === Dynamic Option Handlers ===
-
-    /// Handle dynamic option enable (`:set optionname`)
-    fn handle_dynamic_option_enable(&self, name: &str) {
-        use crate::option::{ChangeSource, OptionChanged, OptionValue};
-
-        if let Some(old_value) = self.option_registry.get(name) {
-            let new_value = OptionValue::Bool(true);
-            match self.option_registry.set(name, new_value.clone()) {
-                Ok(_) => {
-                    tracing::info!(option = %name, "Option enabled");
-                    // Emit change event
-                    self.event_bus.emit(OptionChanged::new(
-                        name,
-                        old_value,
-                        new_value,
-                        ChangeSource::UserCommand,
-                    ));
-                }
-                Err(e) => {
-                    tracing::warn!(option = %name, error = %e, "Failed to enable option");
-                }
-            }
-        } else {
-            tracing::warn!(option = %name, "E518: Unknown option");
-        }
-    }
-
-    /// Handle dynamic option disable (`:set nooptionname`)
-    fn handle_dynamic_option_disable(&self, name: &str) {
-        use crate::option::{ChangeSource, OptionChanged, OptionValue};
-
-        if let Some(old_value) = self.option_registry.get(name) {
-            let new_value = OptionValue::Bool(false);
-            match self.option_registry.set(name, new_value.clone()) {
-                Ok(_) => {
-                    tracing::info!(option = %name, "Option disabled");
-                    // Emit change event
-                    self.event_bus.emit(OptionChanged::new(
-                        name,
-                        old_value,
-                        new_value,
-                        ChangeSource::UserCommand,
-                    ));
-                }
-                Err(e) => {
-                    tracing::warn!(option = %name, error = %e, "Failed to disable option");
-                }
-            }
-        } else {
-            tracing::warn!(option = %name, "E518: Unknown option");
-        }
-    }
-
-    /// Handle dynamic option set (`:set optionname=value`)
-    fn handle_dynamic_option_set(&self, name: &str, value_str: &str) {
-        use crate::option::{ChangeSource, OptionChanged, OptionValue};
-
-        // Get the spec to determine the expected type
-        let Some(spec) = self.option_registry.get_spec(name) else {
-            tracing::warn!(option = %name, "E518: Unknown option");
-            return;
-        };
-
-        // Parse value based on expected type
-        let new_value = match &spec.default {
-            OptionValue::Bool(_) => {
-                // For booleans, accept "true"/"false", "1"/"0", "yes"/"no"
-                match value_str.to_lowercase().as_str() {
-                    "true" | "1" | "yes" | "on" => OptionValue::Bool(true),
-                    "false" | "0" | "no" | "off" => OptionValue::Bool(false),
-                    _ => {
-                        tracing::warn!(option = %name, value = %value_str, "E474: Invalid boolean argument");
-                        return;
-                    }
-                }
-            }
-            OptionValue::Integer(_) => {
-                if let Ok(i) = value_str.parse::<i64>() {
-                    OptionValue::Integer(i)
-                } else {
-                    tracing::warn!(option = %name, value = %value_str, "E521: Number required");
-                    return;
-                }
-            }
-            OptionValue::String(_) => OptionValue::String(value_str.to_string()),
-            OptionValue::Choice { choices, .. } => {
-                if choices.contains(&value_str.to_string()) {
-                    OptionValue::Choice {
-                        value: value_str.to_string(),
-                        choices: choices.clone(),
-                    }
-                } else {
-                    tracing::warn!(
-                        option = %name,
-                        value = %value_str,
-                        valid = %choices.join(", "),
-                        "E474: Invalid choice"
-                    );
-                    return;
-                }
-            }
-        };
-
-        // Get old value for change event
-        let old_value = self
-            .option_registry
-            .get(name)
-            .unwrap_or_else(|| spec.default.clone());
-
-        // Set the value
-        match self.option_registry.set(name, new_value.clone()) {
-            Ok(_) => {
-                tracing::info!(option = %name, value = %value_str, "Option set");
-                // Emit change event
-                self.event_bus.emit(OptionChanged::new(
-                    name,
-                    old_value,
-                    new_value,
-                    ChangeSource::UserCommand,
-                ));
-            }
-            Err(e) => {
-                tracing::warn!(option = %name, value = %value_str, error = %e, "E474: Invalid value");
-            }
-        }
-    }
-
-    /// Handle dynamic option query (`:set optionname?`)
-    fn handle_dynamic_option_query(&self, name: &str) {
-        if let Some(value) = self.option_registry.get(name) {
-            tracing::info!(option = %name, value = %value, "Option value: {name}={value}");
-        } else {
-            tracing::warn!(option = %name, "E518: Unknown option");
-        }
-    }
-
-    /// Handle dynamic option reset (`:set optionname&`)
-    fn handle_dynamic_option_reset(&self, name: &str) {
-        use crate::option::{ChangeSource, OptionChanged};
-
-        // Get the spec to find the default value
-        let Some(spec) = self.option_registry.get_spec(name) else {
-            tracing::warn!(option = %name, "E518: Unknown option");
-            return;
-        };
-
-        let old_value = self
-            .option_registry
-            .get(name)
-            .unwrap_or_else(|| spec.default.clone());
-        let default_value = spec.default;
-
-        match self.option_registry.reset(name) {
-            Ok(_) => {
-                tracing::info!(option = %name, "Option reset to default");
-                // Emit change event
-                self.event_bus.emit(OptionChanged::new(
-                    name,
-                    old_value,
-                    default_value,
-                    ChangeSource::UserCommand,
-                ));
-            }
-            Err(e) => {
-                tracing::warn!(option = %name, error = %e, "Failed to reset option");
-            }
-        }
-    }
 }
