@@ -20,16 +20,19 @@ use reovim_core::{
     bind::{CommandRef, EditModeKind, KeymapScope},
     event_bus::{EventBus, EventResult},
     keys,
-    modd::{ComponentId, EditMode, InsertVariant, VisualVariant},
+    modd::ComponentId,
     plugin::{Plugin, PluginContext, PluginId, PluginStateRegistry},
+    subscribe_state, subscribe_state_mode,
 };
 
 mod command;
+mod file_colors;
 mod node;
 mod provider;
 mod render;
 mod state;
 mod tree;
+mod tree_render;
 mod window;
 
 #[cfg(test)]
@@ -51,7 +54,8 @@ pub mod command_id {
     pub const PAGE_DOWN: CommandId = CommandId::new("explorer_page_down");
     pub const GOTO_FIRST: CommandId = CommandId::new("explorer_goto_first");
     pub const GOTO_LAST: CommandId = CommandId::new("explorer_goto_last");
-    pub const GO_TO_PARENT: CommandId = CommandId::new("explorer_go_to_parent");
+    pub const GO_TO_PARENT: CommandId = CommandId::new("explorer_goto_parent");
+    pub const CHANGE_ROOT: CommandId = CommandId::new("explorer_change_root");
 
     // Tree operations
     pub const TOGGLE_NODE: CommandId = CommandId::new("explorer_toggle_node");
@@ -95,10 +99,10 @@ pub mod command_id {
 
 // Plugin unified command-event types
 pub use command::{
-    ExplorerCancelInput, ExplorerClearFilter, ExplorerClose, ExplorerCloseParent,
-    ExplorerClosePopup, ExplorerConfirmInput, ExplorerCopyPath, ExplorerCreateDir,
-    ExplorerCreateFile, ExplorerCursorDown, ExplorerCursorUp, ExplorerCut, ExplorerDelete,
-    ExplorerExitVisual, ExplorerFocusEditor, ExplorerGoToParent, ExplorerGotoFirst,
+    ExplorerCancelInput, ExplorerChangeRoot, ExplorerClearFilter, ExplorerClose,
+    ExplorerCloseParent, ExplorerClosePopup, ExplorerConfirmInput, ExplorerCopyPath,
+    ExplorerCreateDir, ExplorerCreateFile, ExplorerCursorDown, ExplorerCursorUp, ExplorerCut,
+    ExplorerDelete, ExplorerExitVisual, ExplorerFocusEditor, ExplorerGoToParent, ExplorerGotoFirst,
     ExplorerGotoLast, ExplorerInputBackspace, ExplorerInputChar, ExplorerOpenNode,
     ExplorerPageDown, ExplorerPageUp, ExplorerPaste, ExplorerRefresh, ExplorerRename,
     ExplorerSelectAll, ExplorerShowInfo, ExplorerStartFilter, ExplorerToggle, ExplorerToggleHidden,
@@ -164,22 +168,124 @@ impl Plugin for ExplorerPlugin {
     }
 
     fn subscribe(&self, bus: &EventBus, state: Arc<PluginStateRegistry>) {
-        use reovim_core::{
-            event_bus::core_events::{
-                PluginBackspace, PluginTextInput, RequestFocusChange, RequestModeChange,
-                RequestOpenFile,
-            },
-            modd::{EditMode, ModeState, SubMode},
+        self.subscribe_raw_input(bus, &state);
+        self.subscribe_navigation(bus, &state);
+        self.subscribe_tree_operations(bus, &state);
+        self.subscribe_clipboard(bus, &state);
+        self.subscribe_file_operations(bus, &state);
+        self.subscribe_input_handling(bus, &state);
+        self.subscribe_visual_mode(bus, &state);
+        self.subscribe_focus_visibility(bus, &state);
+        self.subscribe_popup(bus, &state);
+        self.subscribe_settings(bus, &state);
+    }
+}
+
+impl ExplorerPlugin {
+    /// Register and subscribe to explorer settings
+    fn subscribe_settings(&self, bus: &EventBus, state: &Arc<PluginStateRegistry>) {
+        use reovim_core::option::{
+            OptionChanged, OptionSpec, OptionValue, RegisterOption, RegisterSettingSection,
         };
 
-        // Handle text input from runtime (PluginTextInput event)
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<PluginTextInput, _>(100, move |event, ctx| {
-            // Only handle if we're the target
-            if event.target != COMPONENT_ID {
-                return EventResult::NotHandled;
-            }
+        // Register settings section
+        bus.emit(
+            RegisterSettingSection::new("explorer", "File Explorer")
+                .with_description("File tree browser settings")
+                .with_order(110),
+        ); // After core (0-50), before custom (200+)
 
+        // Register options
+        bus.emit(RegisterOption::new(
+            OptionSpec::new(
+                "explorer.enable_colors",
+                "Enable file type coloring",
+                OptionValue::Bool(true),
+            )
+            .with_section("File Explorer")
+            .with_display_order(10),
+        ));
+
+        bus.emit(RegisterOption::new(
+            OptionSpec::new(
+                "explorer.tree_style",
+                "Tree drawing style",
+                OptionValue::String("box_drawing".to_string()),
+            )
+            .with_section("File Explorer")
+            .with_display_order(20),
+        ));
+
+        bus.emit(RegisterOption::new(
+            OptionSpec::new("explorer.show_hidden", "Show hidden files", OptionValue::Bool(false))
+                .with_section("File Explorer")
+                .with_display_order(30),
+        ));
+
+        bus.emit(RegisterOption::new(
+            OptionSpec::new("explorer.show_sizes", "Show file sizes", OptionValue::Bool(false))
+                .with_section("File Explorer")
+                .with_display_order(40),
+        ));
+
+        // Subscribe to option changes
+        let state_clone = Arc::clone(state);
+        bus.subscribe::<OptionChanged, _>(100, move |event, ctx| {
+            match event.name.as_str() {
+                "explorer.enable_colors" => {
+                    if let Some(enabled) = event.new_value.as_bool() {
+                        state_clone.with_mut::<ExplorerState, _, _>(|explorer| {
+                            explorer.enable_colors = enabled;
+                        });
+                        ctx.request_render();
+                    }
+                }
+                "explorer.tree_style" => {
+                    if let Some(style_str) = event.new_value.as_str() {
+                        let tree_style = match style_str {
+                            "none" => crate::state::TreeStyle::None,
+                            "simple" => crate::state::TreeStyle::Simple,
+                            "box_drawing" => crate::state::TreeStyle::BoxDrawing,
+                            _ => crate::state::TreeStyle::BoxDrawing,
+                        };
+                        state_clone.with_mut::<ExplorerState, _, _>(|explorer| {
+                            explorer.tree_style = tree_style;
+                        });
+                        ctx.request_render();
+                    }
+                }
+                "explorer.show_hidden" => {
+                    if let Some(show) = event.new_value.as_bool() {
+                        state_clone.with_mut::<ExplorerState, _, _>(|explorer| {
+                            explorer.show_hidden = show;
+                        });
+                        ctx.request_render();
+                    }
+                }
+                "explorer.show_sizes" => {
+                    if let Some(show) = event.new_value.as_bool() {
+                        state_clone.with_mut::<ExplorerState, _, _>(|explorer| {
+                            explorer.show_sizes = show;
+                        });
+                        ctx.request_render();
+                    }
+                }
+                _ => {}
+            }
+            EventResult::Handled
+        });
+    }
+}
+
+// Event subscription sub-methods
+impl ExplorerPlugin {
+    /// Subscribe to raw text input events (PluginTextInput, PluginBackspace)
+    fn subscribe_raw_input(&self, bus: &EventBus, state: &Arc<PluginStateRegistry>) {
+        use reovim_core::event_bus::core_events::{PluginBackspace, PluginTextInput};
+
+        // Handle text input from runtime (PluginTextInput event)
+        let state_clone = Arc::clone(state);
+        bus.subscribe_targeted::<PluginTextInput, _>(COMPONENT_ID, 100, move |event, ctx| {
             state_clone.with_mut::<ExplorerState, _, _>(|s| {
                 s.input_char(event.c);
             });
@@ -188,100 +294,89 @@ impl Plugin for ExplorerPlugin {
         });
 
         // Handle backspace from runtime (PluginBackspace event)
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<PluginBackspace, _>(100, move |event, ctx| {
-            // Only handle if we're the target
-            if event.target != COMPONENT_ID {
-                return EventResult::NotHandled;
+        // In input mode: delete character from input buffer
+        // In normal mode: navigate to parent directory
+        let state_clone = Arc::clone(state);
+        bus.subscribe_targeted::<PluginBackspace, _>(COMPONENT_ID, 100, move |_event, ctx| {
+            let is_input_mode = state_clone
+                .with::<ExplorerState, _, _>(|s| s.is_input_mode())
+                .unwrap_or(false);
+
+            if is_input_mode {
+                // In input mode, handle as text backspace
+                state_clone.with_mut::<ExplorerState, _, _>(|s| {
+                    s.input_backspace();
+                });
+            } else {
+                // In normal mode, navigate to parent directory
+                state_clone.with_mut::<ExplorerState, _, _>(|s| {
+                    s.go_to_parent();
+                    s.update_scroll();
+                    s.sync_popup();
+                });
             }
-
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.input_backspace();
-            });
             ctx.request_render();
             EventResult::Handled
         });
+    }
 
-        // Navigation events (sync popup with cursor if visible)
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerCursorUp, _>(100, move |event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.move_cursor(-(event.count as isize));
-                s.update_scroll();
-                s.sync_popup();
-            });
-            ctx.request_render();
-            EventResult::Handled
+    /// Subscribe to navigation events (cursor movement, page up/down, goto)
+    fn subscribe_navigation(&self, bus: &EventBus, state: &Arc<PluginStateRegistry>) {
+        subscribe_state!(bus, state, ExplorerCursorUp, ExplorerState, |s, e| {
+            s.move_cursor(-(e.count as isize));
+            s.update_scroll();
+            s.sync_popup();
         });
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerCursorDown, _>(100, move |event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.move_cursor(event.count as isize);
-                s.update_scroll();
-                s.sync_popup();
-            });
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerCursorDown, ExplorerState, |s, e| {
+            s.move_cursor(e.count as isize);
+            s.update_scroll();
+            s.sync_popup();
         });
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerPageUp, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.move_page(s.visible_height, false);
-                s.update_scroll();
-                s.sync_popup();
-            });
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerPageUp, ExplorerState, |s| {
+            s.move_page(s.visible_height, false);
+            s.update_scroll();
+            s.sync_popup();
         });
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerPageDown, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.move_page(s.visible_height, true);
-                s.update_scroll();
-                s.sync_popup();
-            });
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerPageDown, ExplorerState, |s| {
+            s.move_page(s.visible_height, true);
+            s.update_scroll();
+            s.sync_popup();
         });
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerGotoFirst, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.move_to_first();
-                s.update_scroll();
-                s.sync_popup();
-            });
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerGotoFirst, ExplorerState, |s| {
+            s.move_to_first();
+            s.update_scroll();
+            s.sync_popup();
         });
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerGotoLast, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.move_to_last();
-                s.update_scroll();
-                s.sync_popup();
-            });
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerGotoLast, ExplorerState, |s| {
+            s.move_to_last();
+            s.update_scroll();
+            s.sync_popup();
         });
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerGoToParent, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.go_to_parent();
-                s.update_scroll();
-                s.sync_popup();
-            });
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerGoToParent, ExplorerState, |s| {
+            s.go_to_parent();
+            s.update_scroll();
+            s.sync_popup();
         });
 
+        subscribe_state!(bus, state, ExplorerChangeRoot, ExplorerState, |s| {
+            s.change_root_to_current();
+            s.update_scroll();
+            s.sync_popup();
+        });
+    }
+
+    /// Subscribe to tree operation events (open, toggle, refresh, etc.)
+    fn subscribe_tree_operations(&self, bus: &EventBus, state: &Arc<PluginStateRegistry>) {
+        use reovim_core::event_bus::core_events::{RequestFocusChange, RequestOpenFile};
+
         // Open file or toggle directory
-        let state_clone = Arc::clone(&state);
+        let state_clone = Arc::clone(state);
         bus.subscribe::<ExplorerOpenNode, _>(100, move |_event, ctx| {
             tracing::info!("ExplorerPlugin: ExplorerOpenNode received");
 
@@ -322,186 +417,175 @@ impl Plugin for ExplorerPlugin {
         });
 
         // Tree manipulation events
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerToggleNode, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                let _ = s.toggle_current();
-            });
+        subscribe_state!(bus, state, ExplorerToggleNode, ExplorerState, |s| {
+            let _ = s.toggle_current();
+        });
+
+        subscribe_state!(bus, state, ExplorerCloseParent, ExplorerState, |s| {
+            s.collapse_current();
+        });
+
+        subscribe_state!(bus, state, ExplorerRefresh, ExplorerState, |s| {
+            let _ = s.refresh();
+        });
+
+        subscribe_state!(bus, state, ExplorerToggleHidden, ExplorerState, |s| {
+            s.toggle_hidden();
+        });
+
+        subscribe_state!(bus, state, ExplorerToggleSizes, ExplorerState, |s| {
+            s.toggle_sizes();
+        });
+    }
+
+    /// Subscribe to clipboard events (yank, cut, paste, copy path)
+    fn subscribe_clipboard(&self, bus: &EventBus, state: &Arc<PluginStateRegistry>) {
+        subscribe_state!(bus, state, ExplorerYank, ExplorerState, |s| {
+            s.yank_current();
+        });
+
+        subscribe_state!(bus, state, ExplorerCut, ExplorerState, |s| {
+            s.cut_current();
+        });
+
+        subscribe_state!(bus, state, ExplorerPaste, ExplorerState, |s| {
+            let _ = s.paste();
+        });
+
+        // Copy path to clipboard
+        let state_clone = Arc::clone(state);
+        bus.subscribe::<ExplorerCopyPath, _>(100, move |_event, ctx| {
+            use reovim_core::event_bus::core_events::RequestSetRegister;
+
+            let path = state_clone
+                .with::<ExplorerState, _, _>(|s| {
+                    s.current_node()
+                        .map(|n| n.path.to_string_lossy().to_string())
+                })
+                .flatten();
+
+            if let Some(path) = path {
+                // Set both unnamed register and system clipboard
+                ctx.emit(RequestSetRegister {
+                    register: None, // Unnamed register for 'p' paste
+                    text: path.clone(),
+                });
+                ctx.emit(RequestSetRegister {
+                    register: Some('+'), // System clipboard
+                    text: path,
+                });
+
+                state_clone.with_mut::<ExplorerState, _, _>(|s| {
+                    s.close_popup();
+                    s.message = Some("Path copied to clipboard".to_string());
+                });
+            }
+
             ctx.request_render();
             EventResult::Handled
         });
+    }
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerCloseParent, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.collapse_current();
-            });
-            ctx.request_render();
-            EventResult::Handled
-        });
+    /// Subscribe to file operation events (create, rename, delete, filter)
+    fn subscribe_file_operations(&self, bus: &EventBus, state: &Arc<PluginStateRegistry>) {
+        use reovim_core::modd::{EditMode, ModeState, SubMode};
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerRefresh, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                let _ = s.refresh();
-            });
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerToggleHidden, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.toggle_hidden();
-            });
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerToggleSizes, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.toggle_sizes();
-            });
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        // Clipboard events
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerYank, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.yank_current();
-            });
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerCut, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.cut_current();
-            });
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerPaste, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                let _ = s.paste();
-            });
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        // File operation events
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerCreateFile, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
+        subscribe_state_mode!(
+            bus,
+            state,
+            ExplorerCreateFile,
+            ExplorerState,
+            |s| {
                 s.start_create_file();
-            });
-
-            // Enter Interactor sub-mode so characters route to focus handler
-            let mode = ModeState::with_interactor_id_sub_mode(
+            },
+            ModeState::with_interactor_id_sub_mode(
                 COMPONENT_ID,
                 EditMode::Normal,
-                SubMode::Interactor(COMPONENT_ID),
-            );
-            ctx.emit(RequestModeChange { mode });
+                SubMode::Interactor(COMPONENT_ID)
+            )
+        );
 
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerCreateDir, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
+        subscribe_state_mode!(
+            bus,
+            state,
+            ExplorerCreateDir,
+            ExplorerState,
+            |s| {
                 s.start_create_dir();
-            });
-
-            // Enter Interactor sub-mode so characters route to focus handler
-            let mode = ModeState::with_interactor_id_sub_mode(
+            },
+            ModeState::with_interactor_id_sub_mode(
                 COMPONENT_ID,
                 EditMode::Normal,
-                SubMode::Interactor(COMPONENT_ID),
-            );
-            ctx.emit(RequestModeChange { mode });
+                SubMode::Interactor(COMPONENT_ID)
+            )
+        );
 
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerRename, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
+        subscribe_state_mode!(
+            bus,
+            state,
+            ExplorerRename,
+            ExplorerState,
+            |s| {
                 s.start_rename();
-            });
-
-            // Enter Interactor sub-mode so characters route to focus handler
-            let mode = ModeState::with_interactor_id_sub_mode(
+            },
+            ModeState::with_interactor_id_sub_mode(
                 COMPONENT_ID,
                 EditMode::Normal,
-                SubMode::Interactor(COMPONENT_ID),
-            );
-            ctx.emit(RequestModeChange { mode });
+                SubMode::Interactor(COMPONENT_ID)
+            )
+        );
 
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerDelete, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
+        subscribe_state_mode!(
+            bus,
+            state,
+            ExplorerDelete,
+            ExplorerState,
+            |s| {
                 s.start_delete();
-            });
-
-            // Enter Interactor sub-mode for confirmation input
-            let mode = ModeState::with_interactor_id_sub_mode(
+            },
+            ModeState::with_interactor_id_sub_mode(
                 COMPONENT_ID,
                 EditMode::Normal,
-                SubMode::Interactor(COMPONENT_ID),
-            );
-            ctx.emit(RequestModeChange { mode });
+                SubMode::Interactor(COMPONENT_ID)
+            )
+        );
 
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerStartFilter, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
+        subscribe_state_mode!(
+            bus,
+            state,
+            ExplorerStartFilter,
+            ExplorerState,
+            |s| {
                 s.start_filter();
-            });
-
-            // Enter Interactor sub-mode so characters route to focus handler
-            let mode = ModeState::with_interactor_id_sub_mode(
+            },
+            ModeState::with_interactor_id_sub_mode(
                 COMPONENT_ID,
                 EditMode::Normal,
-                SubMode::Interactor(COMPONENT_ID),
-            );
-            ctx.emit(RequestModeChange { mode });
+                SubMode::Interactor(COMPONENT_ID)
+            )
+        );
 
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerClearFilter, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
+        // Exit Interactor sub-mode back to normal explorer mode
+        subscribe_state_mode!(
+            bus,
+            state,
+            ExplorerClearFilter,
+            ExplorerState,
+            |s| {
                 s.clear_filter();
-            });
+            },
+            ModeState::with_interactor_id_and_mode(COMPONENT_ID, EditMode::Normal)
+        );
+    }
 
-            // Exit Interactor sub-mode back to normal explorer mode
-            let mode = ModeState::with_interactor_id_and_mode(COMPONENT_ID, EditMode::Normal);
-            ctx.emit(RequestModeChange { mode });
+    /// Subscribe to input handling events (confirm, cancel, char input)
+    fn subscribe_input_handling(&self, bus: &EventBus, state: &Arc<PluginStateRegistry>) {
+        use reovim_core::{
+            event_bus::core_events::RequestModeChange,
+            modd::{EditMode, ModeState},
+        };
 
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        // Input events
-        let state_clone = Arc::clone(&state);
+        let state_clone = Arc::clone(state);
         bus.subscribe::<ExplorerConfirmInput, _>(100, move |_event, ctx| {
             // Check if popup is visible
             let popup_visible = state_clone
@@ -542,7 +626,7 @@ impl Plugin for ExplorerPlugin {
             EventResult::Handled
         });
 
-        let state_clone = Arc::clone(&state);
+        let state_clone = Arc::clone(state);
         bus.subscribe::<ExplorerCancelInput, _>(100, move |_event, ctx| {
             // Check if popup is visible
             let popup_visible = state_clone
@@ -583,63 +667,40 @@ impl Plugin for ExplorerPlugin {
             EventResult::Handled
         });
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerInputChar, _>(100, move |event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.input_char(event.c);
-            });
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerInputChar, ExplorerState, |s, e| {
+            s.input_char(e.c);
         });
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerInputBackspace, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.input_backspace();
-            });
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerInputBackspace, ExplorerState, |s| {
+            s.input_backspace();
+        });
+    }
+
+    /// Subscribe to visual selection mode events
+    fn subscribe_visual_mode(&self, bus: &EventBus, state: &Arc<PluginStateRegistry>) {
+        subscribe_state!(bus, state, ExplorerVisualMode, ExplorerState, |s| {
+            s.enter_visual_mode();
         });
 
-        // Visual selection events
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerVisualMode, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.enter_visual_mode();
-            });
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerToggleSelect, ExplorerState, |s| {
+            s.toggle_select_current();
         });
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerToggleSelect, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.toggle_select_current();
-            });
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerSelectAll, ExplorerState, |s| {
+            s.select_all();
         });
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerSelectAll, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.select_all();
-            });
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerExitVisual, ExplorerState, |s| {
+            s.exit_visual_mode();
         });
+    }
 
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerExitVisual, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.exit_visual_mode();
-            });
-            ctx.request_render();
-            EventResult::Handled
-        });
+    /// Subscribe to focus and visibility events (toggle, close, focus editor)
+    fn subscribe_focus_visibility(&self, bus: &EventBus, state: &Arc<PluginStateRegistry>) {
+        use reovim_core::event_bus::core_events::RequestFocusChange;
 
         // Toggle explorer visibility
-        let state_clone = Arc::clone(&state);
+        let state_clone = Arc::clone(state);
         bus.subscribe::<ExplorerToggle, _>(100, move |_event, ctx| {
             tracing::info!("ExplorerPlugin: ExplorerToggle received");
 
@@ -685,7 +746,7 @@ impl Plugin for ExplorerPlugin {
         });
 
         // Close explorer and return focus to editor
-        let state_clone = Arc::clone(&state);
+        let state_clone = Arc::clone(state);
         bus.subscribe::<ExplorerClose, _>(100, move |_event, ctx| {
             tracing::info!("ExplorerPlugin: ExplorerClose received");
 
@@ -706,7 +767,6 @@ impl Plugin for ExplorerPlugin {
         });
 
         // Focus editor (without closing explorer)
-        let _state_clone = Arc::clone(&state);
         bus.subscribe::<ExplorerFocusEditor, _>(100, move |_event, ctx| {
             tracing::info!("ExplorerPlugin: ExplorerFocusEditor received");
 
@@ -718,58 +778,16 @@ impl Plugin for ExplorerPlugin {
             ctx.request_render();
             EventResult::Handled
         });
+    }
 
-        // Show file details popup
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerShowInfo, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.show_file_details();
-            });
-            ctx.request_render();
-            EventResult::Handled
+    /// Subscribe to popup events (show info, close popup)
+    fn subscribe_popup(&self, bus: &EventBus, state: &Arc<PluginStateRegistry>) {
+        subscribe_state!(bus, state, ExplorerShowInfo, ExplorerState, |s| {
+            s.show_file_details();
         });
 
-        // Close file details popup
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerClosePopup, _>(100, move |_event, ctx| {
-            state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                s.close_popup();
-            });
-            ctx.request_render();
-            EventResult::Handled
-        });
-
-        // Copy path to clipboard
-        let state_clone = Arc::clone(&state);
-        bus.subscribe::<ExplorerCopyPath, _>(100, move |_event, ctx| {
-            use reovim_core::event_bus::core_events::RequestSetRegister;
-
-            let path = state_clone
-                .with::<ExplorerState, _, _>(|s| {
-                    s.current_node()
-                        .map(|n| n.path.to_string_lossy().to_string())
-                })
-                .flatten();
-
-            if let Some(path) = path {
-                // Set both unnamed register and system clipboard
-                ctx.emit(RequestSetRegister {
-                    register: None, // Unnamed register for 'p' paste
-                    text: path.clone(),
-                });
-                ctx.emit(RequestSetRegister {
-                    register: Some('+'), // System clipboard
-                    text: path,
-                });
-
-                state_clone.with_mut::<ExplorerState, _, _>(|s| {
-                    s.close_popup();
-                    s.message = Some("Path copied to clipboard".to_string());
-                });
-            }
-
-            ctx.request_render();
-            EventResult::Handled
+        subscribe_state!(bus, state, ExplorerClosePopup, ExplorerState, |s| {
+            s.close_popup();
         });
     }
 }
@@ -780,10 +798,23 @@ pub const COMPONENT_ID: ComponentId = ComponentId("explorer");
 #[allow(clippy::unused_self)]
 impl ExplorerPlugin {
     fn register_display_info(&self, ctx: &mut PluginContext) {
+        use reovim_core::highlight::{Color, Style};
+
+        // Orange background for file explorer
+        let orange = Color::Rgb {
+            r: 255,
+            g: 153,
+            b: 51,
+        };
+        let fg = Color::Rgb {
+            r: 33,
+            g: 37,
+            b: 43,
+        };
+        let style = Style::new().fg(fg).bg(orange).bold();
+
         ctx.display_info(COMPONENT_ID)
-            .default(" EXPLORER ", "󰙅 ")
-            .when_mode(&EditMode::Insert(InsertVariant::Standard), " EXPLORER | INSERT ", "󰙅 ")
-            .when_mode(&EditMode::Visual(VisualVariant::Char), " EXPLORER | VISUAL ", "󰙅 ")
+            .default(" EXPLORER ", "󰙅 ", style)
             .register();
     }
 
@@ -795,6 +826,7 @@ impl ExplorerPlugin {
         let _ = ctx.register_command(ExplorerGotoFirst);
         let _ = ctx.register_command(ExplorerGotoLast);
         let _ = ctx.register_command(ExplorerGoToParent);
+        let _ = ctx.register_command(ExplorerChangeRoot);
     }
 
     fn register_tree_commands(&self, ctx: &PluginContext) {
@@ -895,6 +927,16 @@ impl ExplorerPlugin {
             keys!['-'],
             CommandRef::Registered(command_id::GO_TO_PARENT),
         );
+        ctx.bind_key_scoped(
+            explorer_normal.clone(),
+            keys![Backspace],
+            CommandRef::Registered(command_id::GO_TO_PARENT),
+        );
+        ctx.bind_key_scoped(
+            explorer_normal.clone(),
+            keys!['.'],
+            CommandRef::Registered(command_id::CHANGE_ROOT),
+        );
 
         // Tree operations / Input confirmation
         // Enter: Confirm input if in input mode, otherwise open node
