@@ -27,8 +27,10 @@ fn mock_editor_context() -> EditorContext {
         active_window_anchor_y: 0,
         active_window_gutter_width: 0,
         active_window_scroll_y: 0,
+        active_window_height: 0,
         cursor_col: 0,
         cursor_row: 0,
+        active_buffer_content: None,
         color_mode: reovim_core::highlight::ColorMode::TrueColor,
         pending_keys: String::new(),
     }
@@ -310,8 +312,10 @@ fn test_popup_positioned_below_cursor() {
             active_window_anchor_y: 0,
             active_window_gutter_width: 0,
             active_window_scroll_y: 0,
+            active_window_height: 0,
             cursor_col: 0,
             cursor_row: 0,
+            active_buffer_content: None,
             color_mode: reovim_core::highlight::ColorMode::TrueColor,
             pending_keys: String::new(),
         };
@@ -359,8 +363,10 @@ fn test_popup_does_not_exceed_screen_width() {
         active_window_anchor_y: 0,
         active_window_gutter_width: 0,
         active_window_scroll_y: 0,
+        active_window_height: 0,
         cursor_col: 0,
         cursor_row: 0,
+        active_buffer_content: None,
         color_mode: reovim_core::highlight::ColorMode::TrueColor,
         pending_keys: String::new(),
     };
@@ -412,8 +418,10 @@ fn test_popup_clamped_when_cursor_near_bottom() {
         active_window_anchor_y: 0,
         active_window_gutter_width: 0,
         active_window_scroll_y: 0,
+        active_window_height: 0,
         cursor_col: 0,
         cursor_row: 0,
+        active_buffer_content: None,
         color_mode: reovim_core::highlight::ColorMode::TrueColor,
         pending_keys: String::new(),
     };
@@ -463,8 +471,10 @@ fn test_popup_x_position_next_to_explorer() {
             active_window_anchor_y: 0,
             active_window_gutter_width: 0,
             active_window_scroll_y: 0,
+            active_window_height: 0,
             cursor_col: 0,
             cursor_row: 0,
+            active_buffer_content: None,
             color_mode: reovim_core::highlight::ColorMode::TrueColor,
             pending_keys: String::new(),
         };
