@@ -14,6 +14,7 @@
 
 mod command;
 mod document;
+mod health;
 mod hover;
 mod manager;
 mod picker;
@@ -58,7 +59,7 @@ pub use {
 // Re-export events and commands for external use
 pub use command::{
     LspGotoDefinition, LspGotoDefinitionCommand, LspGotoReferences, LspGotoReferencesCommand,
-    LspHoverDismiss, LspShowHover, LspShowHoverCommand,
+    LspHoverDismiss, LspLogOpen, LspShowHover, LspShowHoverCommand,
 };
 
 /// LSP integration plugin.
@@ -179,6 +180,79 @@ impl Plugin for LspPlugin {
 
     #[allow(clippy::too_many_lines)]
     fn subscribe(&self, bus: &EventBus, state: Arc<PluginStateRegistry>) {
+        // Capture tokio runtime handle for use in EventBus handlers
+        // (EventBus handlers run on std::thread, not tokio runtime - see #120)
+        let rt_handle = tokio::runtime::Handle::current();
+
+        // Register LSP health check with health-check plugin
+        {
+            use reovim_plugin_health_check::RegisterHealthCheck;
+
+            let manager = Arc::clone(&self.manager);
+            bus.emit(RegisterHealthCheck::new(Arc::new(health::LspHealthCheck::new(manager))));
+        }
+
+        // Register :LspLog ex-command
+        {
+            use reovim_core::{
+                command_line::ExCommandHandler,
+                event_bus::{DynEvent, core_events::RegisterExCommand},
+            };
+
+            bus.emit(RegisterExCommand::new(
+                "LspLog",
+                ExCommandHandler::ZeroArg {
+                    event_constructor: || DynEvent::new(command::LspLogOpen),
+                    description: "Open the LSP log file",
+                },
+            ));
+        }
+
+        // Handle LspLogOpen - find and open the LSP log file
+        bus.subscribe::<command::LspLogOpen, _>(100, move |_event, ctx| {
+            use reovim_core::event_bus::core_events::RequestOpenFile;
+
+            // Find the latest LSP log file in the data directory
+            let data_dir = dirs::data_local_dir()
+                .unwrap_or_else(|| PathBuf::from("."))
+                .join("reovim");
+
+            // Find latest lsp-*.log file
+            if let Ok(entries) = std::fs::read_dir(&data_dir) {
+                let mut lsp_logs: Vec<_> = entries
+                    .filter_map(std::result::Result::ok)
+                    .filter(|e| {
+                        e.file_name().to_string_lossy().starts_with("lsp-")
+                            && e.file_name().to_string_lossy().ends_with(".log")
+                    })
+                    .collect();
+
+                // Sort by modification time (newest first)
+                lsp_logs.sort_by(|a, b| {
+                    b.metadata()
+                        .and_then(|m| m.modified())
+                        .unwrap_or(std::time::SystemTime::UNIX_EPOCH)
+                        .cmp(
+                            &a.metadata()
+                                .and_then(|m| m.modified())
+                                .unwrap_or(std::time::SystemTime::UNIX_EPOCH),
+                        )
+                });
+
+                if let Some(latest) = lsp_logs.first() {
+                    let log_path = latest.path();
+                    info!("Opening LSP log: {}", log_path.display());
+                    ctx.emit(RequestOpenFile { path: log_path });
+                } else {
+                    warn!("No LSP log files found in {}", data_dir.display());
+                }
+            } else {
+                warn!("Could not read data directory: {}", data_dir.display());
+            }
+
+            EventResult::Handled
+        });
+
         // Handle file open - register document and schedule sync for render stage
         // Note: We don't send didOpen here directly. The render stage handles it
         // via schedule_immediate_sync() to avoid version synchronization bugs.
@@ -257,6 +331,7 @@ impl Plugin for LspPlugin {
         {
             let state = Arc::clone(&state);
             let event_sender = bus.sender();
+            let rt_handle = rt_handle.clone();
             bus.subscribe::<LspGotoDefinition, _>(100, move |event, _ctx| {
                 info!(
                     buffer_id = event.buffer_id,
@@ -294,9 +369,10 @@ impl Plugin for LspPlugin {
                     );
 
                     // Spawn async task to handle response
+                    // Use captured rt_handle since EventBus handlers run on std::thread (#120)
                     let sender = event_sender.clone();
                     let state_clone = Arc::clone(&state);
-                    tokio::spawn(async move {
+                    rt_handle.spawn(async move {
                         match rx.await {
                             Ok(Ok(Some(response))) => {
                                 // Extract all locations from response
@@ -363,6 +439,7 @@ impl Plugin for LspPlugin {
         {
             let state = Arc::clone(&state);
             let event_sender = bus.sender();
+            let rt_handle = rt_handle.clone();
             bus.subscribe::<LspGotoReferences, _>(100, move |event, _ctx| {
                 let buffer_id = event.buffer_id;
                 let line = event.line;
@@ -394,9 +471,10 @@ impl Plugin for LspPlugin {
                     );
 
                     // Spawn async task to handle response
+                    // Use captured rt_handle since EventBus handlers run on std::thread (#120)
                     let state_clone = Arc::clone(&state);
                     let sender = event_sender.clone();
-                    tokio::spawn(async move {
+                    rt_handle.spawn(async move {
                         match rx.await {
                             Ok(Ok(Some(locations))) => {
                                 info!(count = locations.len(), "LSP: references found");
@@ -438,6 +516,7 @@ impl Plugin for LspPlugin {
         // Handle show hover command
         {
             let state = Arc::clone(&state);
+            // Last use of rt_handle - no clone needed (moved into closure)
             bus.subscribe::<LspShowHover, _>(100, move |event, _ctx| {
                 let buffer_id = event.buffer_id;
                 let line = event.line;
@@ -469,8 +548,9 @@ impl Plugin for LspPlugin {
                     );
 
                     // Spawn async task to handle response
+                    // Use captured rt_handle since EventBus handlers run on std::thread (#120)
                     let state_clone = Arc::clone(&state);
-                    tokio::spawn(async move {
+                    rt_handle.spawn(async move {
                         match rx.await {
                             Ok(Ok(Some(hover))) => {
                                 // Extract hover text content
@@ -603,6 +683,66 @@ impl Plugin for LspPlugin {
                 }
             });
         }
+
+        // Handle virtual text option changes
+        {
+            use reovim_core::option::{OptionChanged, OptionValue};
+
+            let state = Arc::clone(&state);
+            bus.subscribe::<OptionChanged, _>(100, move |event, _ctx| {
+                let name = event.name.as_str();
+
+                // Only handle virtual_text_* options
+                if !name.starts_with("virtual_text") {
+                    return EventResult::Handled;
+                }
+
+                state.with_mut::<Arc<SharedLspManager>, _, _>(|manager| {
+                    manager.with_mut(|m| {
+                        let config = &mut m.virtual_text_config;
+                        match name {
+                            "virtual_text" => {
+                                if let OptionValue::Bool(enabled) = &event.new_value {
+                                    config.enabled = *enabled;
+                                    debug!("LSP: virtual_text enabled = {}", enabled);
+                                }
+                            }
+                            "virtual_text_prefix" => {
+                                if let OptionValue::String(prefix) = &event.new_value {
+                                    config.prefix.clone_from(prefix);
+                                    debug!("LSP: virtual_text prefix = {:?}", prefix);
+                                }
+                            }
+                            "virtual_text_max_length" => {
+                                if let OptionValue::Integer(len) = &event.new_value {
+                                    #[allow(
+                                        clippy::cast_possible_truncation,
+                                        clippy::cast_sign_loss
+                                    )]
+                                    {
+                                        config.max_length = (*len).clamp(10, 200) as u16;
+                                    }
+                                    debug!("LSP: virtual_text max_length = {}", len);
+                                }
+                            }
+                            "virtual_text_show" => {
+                                if let OptionValue::String(mode) = &event.new_value {
+                                    config.show_mode = match mode.as_str() {
+                                        "highest" => manager::VirtualTextShowMode::Highest,
+                                        "all" => manager::VirtualTextShowMode::All,
+                                        _ => manager::VirtualTextShowMode::First,
+                                    };
+                                    debug!("LSP: virtual_text show_mode = {:?}", config.show_mode);
+                                }
+                            }
+                            _ => {}
+                        }
+                    });
+                });
+
+                EventResult::NeedsRender
+            });
+        }
     }
 
     #[allow(clippy::too_many_lines)]
@@ -610,7 +750,7 @@ impl Plugin for LspPlugin {
         &self,
         bus: &EventBus,
         state: Arc<PluginStateRegistry>,
-        event_tx: Option<tokio::sync::mpsc::Sender<reovim_core::event::InnerEvent>>,
+        event_tx: Option<tokio::sync::mpsc::Sender<reovim_core::event::RuntimeEvent>>,
     ) {
         // Store event_tx in manager for triggering re-renders
         if let Some(tx) = event_tx {
