@@ -22,7 +22,7 @@ use std::{any::TypeId, sync::Arc};
 
 use reovim_core::{
     bind::{CommandRef, EditModeKind, KeyMapInner, KeymapScope, SubModeKind},
-    display::{DisplayInfo, EditModeKey},
+    display::DisplayInfo,
     frame::FrameBuffer,
     highlight::Theme,
     keys,
@@ -534,17 +534,22 @@ impl Plugin for MicroscopePlugin {
 
     fn build(&self, ctx: &mut PluginContext) {
         // Register display info for status line
-        ctx.register_display(COMPONENT_ID, DisplayInfo::new(" MICROSCOPE ", "󰍉 "));
-        ctx.register_component_mode_display(
-            COMPONENT_ID,
-            EditModeKey::Normal,
-            DisplayInfo::new(" MICROSCOPE ", "󰍉 "),
-        );
-        ctx.register_component_mode_display(
-            COMPONENT_ID,
-            EditModeKey::Insert,
-            DisplayInfo::new(" MICROSCOPE | INSERT ", "󰍉 "),
-        );
+        use reovim_core::highlight::{Color, Style};
+
+        // Blue background for search/picker
+        let blue = Color::Rgb {
+            r: 97,
+            g: 175,
+            b: 239,
+        };
+        let fg = Color::Rgb {
+            r: 33,
+            g: 37,
+            b: 43,
+        };
+        let style = Style::new().fg(fg).bg(blue).bold();
+
+        ctx.register_display(COMPONENT_ID, DisplayInfo::new(" MICROSCOPE ", "󰍉 ", style));
 
         // Register picker opening commands
         let _ = ctx.register_command(MicroscopeFindFiles);
@@ -857,7 +862,7 @@ impl Plugin for MicroscopePlugin {
                     PluginBackspace, PluginTextInput, RequestFocusChange, RequestModeChange,
                 },
             },
-            modd::{EditMode, ModeState, SubMode},
+            modd::{EditMode, ModeState},
         };
 
         // Helper function to load preview for the currently selected item
@@ -941,12 +946,7 @@ impl Plugin for MicroscopePlugin {
 
         // Handle text input from runtime (PluginTextInput event)
         let state_clone = Arc::clone(&state);
-        bus.subscribe::<PluginTextInput, _>(100, move |event, ctx| {
-            // Only handle if target is microscope and microscope is active
-            if event.target != COMPONENT_ID {
-                return EventResult::NotHandled;
-            }
-
+        bus.subscribe_targeted::<PluginTextInput, _>(COMPONENT_ID, 100, move |event, ctx| {
             let is_active = state_clone
                 .with::<MicroscopeState, _, _>(|s| s.active)
                 .unwrap_or(false);
@@ -964,12 +964,7 @@ impl Plugin for MicroscopePlugin {
 
         // Handle backspace from runtime (PluginBackspace event)
         let state_clone = Arc::clone(&state);
-        bus.subscribe::<PluginBackspace, _>(100, move |event, ctx| {
-            // Only handle if we're the target
-            if event.target != COMPONENT_ID {
-                return EventResult::NotHandled;
-            }
-
+        bus.subscribe_targeted::<PluginBackspace, _>(COMPONENT_ID, 100, move |_event, ctx| {
             let is_active = state_clone
                 .with::<MicroscopeState, _, _>(|s| s.active)
                 .unwrap_or(false);
@@ -1233,12 +1228,7 @@ impl Plugin for MicroscopePlugin {
             });
 
             // Enter Interactor sub-mode so characters route to PluginTextInput handler
-            let mode = ModeState::with_interactor_id_sub_mode(
-                COMPONENT_ID,
-                EditMode::Normal,
-                SubMode::Interactor(COMPONENT_ID),
-            );
-            ctx.emit(RequestModeChange { mode });
+            ctx.enter_interactor_mode(COMPONENT_ID);
 
             ctx.request_render();
             EventResult::Handled
