@@ -25,10 +25,25 @@
 (fenced_code_block) @decoration.code_block
 (indented_code_block) @decoration.code_block
 
+; Fenced code block language icon - capture opening fence and language
+(fenced_code_block
+  (fenced_code_block_delimiter) @decoration.code_fence_open
+  (info_string
+    (language) @decoration.code_lang))
+
 ; Tables (pipe tables)
 (pipe_table) @decoration.table
 (pipe_table_header) @decoration.table.header
 (pipe_table_delimiter_row) @decoration.table.delimiter
 
+; Horizontal rules (thematic breaks) for visual enhancement
+(thematic_break) @decoration.horizontal_rule
+
+; Blockquotes for styling
+(block_quote) @decoration.blockquote
+(block_quote_marker) @decoration.blockquote.marker
+; Continuation markers on subsequent lines of blockquotes (only inside block_quote)
+(block_quote (paragraph (inline (block_continuation) @decoration.blockquote.marker)))
+
 ; NOTE: Inline elements (emphasis, strong_emphasis, inline_link) are in
 ; INLINE_LANGUAGE, not LANGUAGE. See treesitter/grammar.rs for dual-grammar support.
