@@ -4,8 +4,8 @@ use reovim_core::{
     buffer::{Buffer, TextOps},
     content::WindowContentSource,
     screen::{
-        Position,
-        window::{Anchor, LineNumber, Window},
+        WindowId, WindowRect,
+        window::{LineNumber, SignColumnMode, Viewport, Window, WindowConfig},
     },
 };
 
@@ -23,21 +23,18 @@ pub fn create_buffer(lines: usize) -> Buffer {
 /// Create a window with typical editor dimensions
 pub fn create_window(height: u16) -> Window {
     Window {
-        id: 0,
-        source: WindowContentSource::FileBuffer {
-            buffer_id: 0,
-            buffer_anchor: Anchor { x: 0, y: 0 },
-        },
-        anchor: Anchor { x: 0, y: 0 },
-        width: 120,
-        height,
+        id: WindowId::new(0),
+        source: WindowContentSource::FileBuffer { buffer_id: 0 },
+        bounds: WindowRect::new(0, 0, 120, height),
         z_order: 0,
         is_active: true,
         is_floating: false,
-        line_number: Some(LineNumber::default()),
-        scrollbar_enabled: false,
-        cursor: Position { x: 0, y: 0 },
-        desired_col: None,
-        border_config: None,
+        viewport: Viewport::default(),
+        config: WindowConfig {
+            line_number: Some(LineNumber::default()),
+            sign_column_mode: SignColumnMode::No,
+            scrollbar_enabled: false,
+            border_config: None,
+        },
     }
 }
